from ..file import File
from ..file_folder import FileFolder
from ...types.he import He


class Folder(FileFolder):
    def __init__(self, u: He, path: str): ...
    def __new__(cls, u: He, path: str): ...
    def is_exists(self)-> bool: ...
    def is_unexists(self)-> bool: ...
    def create(self)-> None: ...
    def child_file(self, name: str | list[str]) -> File: ...
    def child_folder(self, name: str | list[str]) -> Folder: ...

    def search_children_names(self)-> list[str]: ...
    def search_children_file_names(self)-> list[str]: ...
    def search_children_folder_names(self)-> list[str]: ...
    def search_usable_children_names(self)-> list[str]: ...
    def search_usable_children_file_names(self)-> list[str]: ...
    def search_usable_children_folder_names(self)-> list[str]: ...

    def search_children_paths(self)-> list[str]: ...
    def search_children_file_paths(self)-> list[str]: ...
    def search_children_folder_paths(self)-> list[str]: ...
    def search_usable_children_paths(self)-> list[str]: ...
    def search_usable_children_file_paths(self)-> list[str]: ...
    def search_usable_children_folder_paths(self)-> list[str]: ...

    def search_children_files(self)-> list[File]: ...
    def search_children_folders(self)-> list[Folder]: ...
    def search_usable_children_files(self)-> list[File]: ...
    def search_usable_children_folders(self)-> list[Folder]: ...
    def search_children(self)-> list[File | Folder]: ...
    def search_usable_children(self)-> list[File | Folder]: ...
    def copyto(self, target_folder: str | Folder): ...



from .....functions.decorators import parse_self_item_value, set_self_item_value_by_setter_with_agr
from ...configs import Configs as ModConfigs


class Configs(ModConfigs):
    def __init__(self, system):
        super().__init__(system)
        self.update({
            "server": "flask",

            "session_expiry": 86400,

            'language': self.__sc__['language'],
            'host': '0.0.0.0',
            'port': 5000,
            'module': 'hebill.website',

            "mysql_host": "",
            "mysql_port": 3306,
            "mysql_user": "",
            "mysql_password": "",
            "mysql_dbname": "",
            "mysql_prefix": "",

            "mysql_max_connections": 30,
            "mysql_connect_timeout": 5,
            "mysql_column_key_sn": "_sn",
            "mysql_column_key_id": "_id",
            "mysql_column_key_az": "_az",
            "mysql_column_key_kw": "_kw",
            "mysql_column_key_ur": "_ur",
            "mysql_column_key_dc": "_dc",
            "mysql_column_key_du": "_du",
            "mysql_data_admin_id": "_admin",

            'webmod_page_content_sign_in': 'sign_in',
            'webmod_page_content_sign_out': 'sign_out',
            'webmod_page_content_sign_up': 'sign_up',
            'webmod_page_content_sign_reset': 'sign_reset',
            'webmod_page_content_about': 'about',
            'webmod_page_content_contact': 'contact',
            'webmod_page_content_admin': 'admin',

            'webmod_page_layout': '',

            'webmod_page_block_1010': '',
            'webmod_page_block_1020': '',
            'webmod_page_block_1030': '',
            'webmod_page_block_1040': '',
            'webmod_page_block_1050': '',
            'webmod_page_block_1060': '',
            'webmod_page_block_1070': '',
            'webmod_page_block_1080': '',
            'webmod_page_block_1090': '',
            'webmod_page_block_1100': '',

            'webmod_page_block_navbar': '',
            'webmod_page_block_breadcrumb': '',
            'webmod_page_block_navigations': '',
            'webmod_page_block_messages': '',
            'webmod_page_block_footer': '',
            'webmod_page_block_debugs': '',

            'html_output_minimized': False,
            'html_output_unspaced': False,
            'html_output_comments': True,
            'html_output_indentation': '\t',
            'html_title_delimiter': '/',
            'html_title_ascending': True,

            "url_root_path": "",

            "url_key_language": "l",
            "url_key_mode": "t",
            "url_key_module": "m",
            "url_key_arguments": "a",
            "url_key_keyword": "k",
            "url_key_page": "n",
            "url_key_orders": "o",
            "url_key_filters": "f",
            "url_key_items_per_page": "i",
            "url_key_pages_per_group": "g",
            "url_key_security": "s",
            "url_key_reference": "r",

            "pagination_items_per_page": 25,
            "pagination_pages_per_group": 10,

            "logo": "",
        })

    @property
    @parse_self_item_value()
    def language(self): return
    @language.setter
    @set_self_item_value_by_setter_with_agr(value_type='lan')
    def language(self, value): pass

    @property
    @parse_self_item_value()
    def session_expiry(self): return
    @session_expiry.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def session_expiry(self, value): pass

    @property
    @parse_self_item_value()
    def host(self): return
    @host.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def host(self, value): pass

    @property
    @parse_self_item_value()
    def port(self): return
    @port.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def port(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_host(self): return
    @mysql_host.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_host(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_port(self): return
    @mysql_port.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def mysql_port(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_user(self): return
    @mysql_user.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_user(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_password(self): return
    @mysql_password.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_password(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_dbname(self): return
    @mysql_dbname.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_dbname(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_prefix(self): return
    @mysql_prefix.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_prefix(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_max_connections(self): return
    @mysql_max_connections.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def mysql_max_connections(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_connect_timeout(self): return
    @mysql_connect_timeout.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def mysql_connect_timeout(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_column_key_sn(self): return
    @mysql_column_key_sn.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_column_key_sn(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_column_key_id(self): return
    @mysql_column_key_id.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_column_key_id(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_column_key_az(self): return
    @mysql_column_key_az.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_column_key_az(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_column_key_kw(self): return
    @mysql_column_key_kw.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_column_key_kw(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_column_key_ur(self): return
    @mysql_column_key_ur.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_column_key_ur(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_column_key_dc(self): return
    @mysql_column_key_dc.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_column_key_dc(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_column_key_du(self): return
    @mysql_column_key_du.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_column_key_du(self, value): pass

    @property
    @parse_self_item_value()
    def mysql_data_admin_id(self): return
    @mysql_data_admin_id.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mysql_data_admin_id(self, value): pass

    @property
    @parse_self_item_value()
    def module(self): return
    @module.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def module(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_content_sign_in(self): return
    @webmod_page_content_sign_in.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_content_sign_in(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_content_sign_out(self): return
    @webmod_page_content_sign_out.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_content_sign_out(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_content_sign_up(self): return
    @webmod_page_content_sign_up.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_content_sign_up(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_content_sign_reset(self): return
    @webmod_page_content_sign_reset.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_content_sign_reset(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_content_admin(self): return
    @webmod_page_content_admin.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_content_admin(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_content_about(self): return
    @webmod_page_content_about.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_content_about(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_content_contact(self): return
    @webmod_page_content_contact.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_content_contact(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_layout(self): return
    @webmod_page_layout.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_layout(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_1010(self): return
    @webmod_page_block_1010.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_1010(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_1020(self): return
    @webmod_page_block_1020.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_1020(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_1030(self): return
    @webmod_page_block_1030.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_1030(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_1040(self): return
    @webmod_page_block_1040.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_1040(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_1050(self): return
    @webmod_page_block_1050.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_1050(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_1060(self): return
    @webmod_page_block_1060.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_1060(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_1070(self): return
    @webmod_page_block_1070.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_1070(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_1080(self): return
    @webmod_page_block_1080.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_1080(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_1090(self): return
    @webmod_page_block_1090.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_1090(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_1100(self): return
    @webmod_page_block_1100.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_1100(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_navbar(self): return
    @webmod_page_block_navbar.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_navbar(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_breadcrumb(self): return
    @webmod_page_block_breadcrumb.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_breadcrumb(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_navigations(self): return
    @webmod_page_block_navigations.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_navigations(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_messages(self): return
    @webmod_page_block_messages.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_messages(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_footer(self): return
    @webmod_page_block_footer.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_footer(self, value): pass

    @property
    @parse_self_item_value()
    def webmod_page_block_debugs(self): return
    @webmod_page_block_debugs.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def webmod_page_block_debugs(self, value): pass

    @property
    @parse_self_item_value()
    def html_output_minimized(self): return
    @html_output_minimized.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def html_output_minimized(self, value): pass

    @property
    @parse_self_item_value()
    def html_output_indentation(self): return
    @html_output_indentation.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def html_output_indentation(self, value): pass

    @property
    @parse_self_item_value()
    def html_output_comments(self): return
    @html_output_comments.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def html_output_comments(self, value): pass

    @property
    @parse_self_item_value()
    def html_output_unspaced(self): return
    @html_output_unspaced.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def html_output_unspaced(self, value): pass

    @property
    @parse_self_item_value()
    def html_title_delimiter(self): return
    @html_title_delimiter.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def html_title_delimiter(self, value): pass

    @property
    @parse_self_item_value()
    def html_title_ascending(self): return
    @html_title_ascending.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def html_title_ascending(self, value): pass

    @property
    @parse_self_item_value()
    def url_root_path(self): return
    @url_root_path.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_root_path(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_language(self): return
    @url_key_language.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_language(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_mode(self): return
    @url_key_mode.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_mode(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_module(self): return
    @url_key_module.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_module(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_arguments(self): return
    @url_key_arguments.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_arguments(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_keyword(self): return
    @url_key_keyword.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_keyword(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_page(self): return
    @url_key_page.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_page(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_orders(self): return
    @url_key_orders.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_orders(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_filters(self): return
    @url_key_filters.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_filters(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_items_per_page(self): return
    @url_key_items_per_page.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_items_per_page(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_pages_per_group(self): return
    @url_key_pages_per_group.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_pages_per_group(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_security(self): return
    @url_key_security.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_security(self, value): pass

    @property
    @parse_self_item_value()
    def url_key_reference(self): return
    @url_key_reference.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def url_key_reference(self, value): pass

    @property
    @parse_self_item_value()
    def pagination_items_per_page(self): return
    @pagination_items_per_page.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def pagination_items_per_page(self, value): pass

    @property
    @parse_self_item_value()
    def pagination_pages_per_group(self): return
    @pagination_pages_per_group.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def pagination_pages_per_group(self, value): pass

    @property
    @parse_self_item_value()
    def logo(self): return
    @logo.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def logo(self, value): pass



from .menus import Menus
from .menus.menu import Menu


class Dropdown(dict):
    @property
    def title(self): ...
    @title.setter
    def title(self, value): ...

    @property
    def menus(self)-> Menus | list[Menu]: ...
    def is_menu(self)-> bool: ...
    def is_dropdown(self)-> bool: ...



from ..comment import Comment
from ..script import Script
from ..text import Text
from ... import Htmdoc
from ...__tags_creater__ import TagsCreater
from ...node import Node


class Wrap(Node, TagsCreater):
    def __init__(self, u: Htmdoc | Wrap, content=None): ...
    @property
    def juniors(self)-> dict[int, Node]: ...

    def comment(self, content: str = None)-> Comment: ...
    def script(self, content: str = None)-> Script: ...
    def text(self, content: str = None)-> Text: ...
    def wrap(self, content=None, associated:Node=None)-> Wrap: ...

    def append(self, element=None)-> Node | list[Node] | None: ...

    def ___anchor_to___(self, node: Node): ...
    @property
    def anchored(self)-> Node | None: ...
    def is_anchored(self)-> bool: ...



from .....functions.decorators import set_self_item_value_by_setter_with_agr, parse_self_item_value
from ...configs import Configs as ModConfigs


class Configs(ModConfigs):
    def __init__(self, u):
        super().__init__(u, {
            'language': 'gb',
            'output_minimized': False,
            'output_unspaced': False,
            'output_comments': True,
            'output_indentation': '\t',
            'title_delimiter': '>',
            'title_ascending': True,
        })

    @property
    @parse_self_item_value()
    def language(self): return
    @language.setter
    @set_self_item_value_by_setter_with_agr(value_type='lan')
    def language(self, value): pass

    @property
    @parse_self_item_value()
    def output_minimized(self): return
    @output_minimized.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def output_minimized(self, value): pass

    @property
    @parse_self_item_value()
    def output_unspaced(self): return
    @output_unspaced.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def output_unspaced(self, value): pass

    @property
    @parse_self_item_value()
    def output_comments(self): return
    @output_comments.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def output_comments(self, value): pass

    @property
    @parse_self_item_value()
    def output_indentation(self): return
    @output_indentation.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def output_indentation(self, value): pass

    @property
    @parse_self_item_value()
    def title_delimiter(self): return
    @title_delimiter.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def title_delimiter(self, value): pass

    @property
    @parse_self_item_value()
    def title_ascending(self): return
    @title_ascending.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def title_ascending(self, value): pass



from io import BytesIO

from ...types.he import He
from ..file_folder import FileFolder


class File(FileFolder):
    def __init__(self, u: He, path: str): ...

    @property
    def base_name(self)-> str: ...
    @property
    def extension_name(self)-> str: ...
    def is_exists(self)-> bool: ...
    def is_unexists(self)-> bool: ...
    def parse_json(self)-> list | dict | None: ...
    def read_content(self)-> str: ...
    def read_lines(self)-> list[str]: ...
    def read_bytes(self)-> bytes: ...
    def save_content(self, content: str)-> None: ...
    def save_json(self, data: dict)-> None: ...
    def copy_to(self, path: File | str)-> bool: ...
    def copy_from(self, path: File | str)-> bool: ...
    def save_io(self, io: BytesIO)-> bool: ...



from ....functions.builtins import parse_mod_path_by_pyobj, parse_mod_path_by_pycls
from ....functions.decorators import parse_self_x_item_value, parse_self_x_item_value_or_set_by_func
from ...types.he import He
from ...types.hs import Hs


class ClsType(Hs):
    def __new__(cls, u, pycls: He | type):
        if isinstance(pycls, He):
            obj = Hs.__new__(cls, u, parse_mod_path_by_pyobj(pycls))
        elif issubclass(pycls, He):
            obj = Hs.__new__(cls, u, parse_mod_path_by_pycls(pycls))
        else:
            raise ValueError(f'ClsType 的参数 pycls 只允许： He | type(He)')
        return obj

    def __init__(self, u, pycls: He | type):
        if isinstance(pycls, He):
            Hs.__init__(self, u, parse_mod_path_by_pyobj(pycls))
            self.__x__['pycls'] = pycls.__class__
            self.__x__['pymod'] = pycls.__module__
        elif issubclass(pycls, He):
            Hs.__init__(self, u, parse_mod_path_by_pycls(pycls))
            self.__x__['pycls'] = pycls
            self.__x__['pymod'] = pycls.__module__
        else:
            raise ValueError(f'ClsType 的参数 pycls 只允许： He | type(He)')

    @property
    @parse_self_x_item_value()
    def pymod(self): return
    @property
    @parse_self_x_item_value()
    def pycls(self): return
    @property
    def mod_path(self): return self.mod_type.mod_path
    @property
    @parse_self_x_item_value_or_set_by_func()
    def mod_type(self): return self.__sm__.mod_type(self.pycls)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def tree_pyclses(self): return [i for i in self.pycls.mro() if issubclass(i, He)]
    @property
    @parse_self_x_item_value_or_set_by_func()
    def tree_cls_types(self): return [self.__sm__.cls_type(i) for i in self.tree_pyclses]

    def has_function(self, name): return callable(self.pycls.__dict__.get(name))
    def has_local_function(self, name): return callable(getattr(self.pycls, name, None))

    @property
    def folder(self): return self.mod_type.folder
    @property
    def data_folder(self): return self.mod_type.data_folder
    @property
    def data_configs_folder(self): return self.mod_type.data_configs_folder
    @property
    def data_configs_file(self): return self.mod_type.data_configs_file
    @property
    def logs_folder(self): return self.mod_type.logs_folder
    @property
    def data_multilinguals_folder(self): return self.mod_type.data_multilinguals_folder
    def data_multilinguals_file(self, language: str): return self.mod_type.data_multilinguals_file(language)
    @property
    def data_database_tables_folder(self): return self.mod_type.data_database_tables_folder
    @property
    def data_database_data_folder(self): return self.mod_type.data_database_data_folder

    @property
    @parse_self_x_item_value_or_set_by_func()
    def multilinguals(self): return self.__sm__.multilinguals(self.pycls, He, hooked_configs=self.__sc__)
    def ml(self, code, language=None, replace=None, return_code=None): return self.multilinguals.ml(code, language, replace, return_code)
    def ml_name(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_name(language, replace, return_code)
    def ml_abbreviation(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_abbreviation(language, replace, return_code)
    def ml_title(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_title(language, replace, return_code)
    def ml_description(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_description(language, replace, return_code)
    def ml_link_title(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_link_title(language, replace, return_code)
    def ml_page_title(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_page_title(language, replace, return_code)
    def ml_breadcrumb_title(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_breadcrumb_title(language, replace, return_code)




from .core import Image
from .core import (IconClose, IconDemo, IconDocument, IconHelp, IconHome, IconIcon, IconInfo, IconLangauge,
                   IconMaximize, IconMinimize, IconMore, IconNormalize, IconQuite, IconSetting, IconUser, IconHebill,
                   IconYangs)



from ..folder import Folder
from ....functions.decorators import parse_self_x_item_value, parse_self_x_item_value_or_set_by_func, \
    parse_self_item_value_or_set_by_instance_he_by_module_name
from ...types.he import He
from ...types.hd import Hd


class Multilinguals(Hd):
    def __init__(self, u, end_pycls=None, start_pycls=None, folder=None, hooked_configs=None, hooked_config_name=None):
        super().__init__(u)
        if folder:
            if isinstance(folder, Folder): self.__x__['folder'] = folder
            else:  self.__x__['folder'] = self.__sm__.folder(folder)
        elif end_pycls:
            start_pycls = start_pycls if start_pycls else He
            if not issubclass(end_pycls, He): raise RuntimeError('senior_class 必须是继承自 Heobj 的类或者实例')
            if not issubclass(end_pycls, start_pycls): raise RuntimeError('end_pycls 必须是继承自 start_pycls 的类或者实例')
            for cls in end_pycls.mro():
                if issubclass(cls, start_pycls):
                    self.___tree_cls_types___.append(self.__sm__.cls_type(cls))
        if hooked_configs: self.__x__['hooked_configs'] = hooked_configs
        if hooked_config_name: self.__x__['hooked_config_name'] = hooked_config_name

    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___tree_cls_types___(self): return []
    @property
    @parse_self_x_item_value()
    def ___hooked_configs___(self): return
    @property
    @parse_self_x_item_value()
    def ___hooked_config_name___(self): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def folder(self): return self.___tree_cls_types___[0].data_multilinguals_folder

    @parse_self_item_value_or_set_by_instance_he_by_module_name(dict_item_key_by_arg=0)
    def package(self, language): return

    @property
    def default(self):
        if self.___hooked_configs___ is not None:
            if (l:=self.___hooked_configs___.___defaults___.get(self.___hooked_config_name___)) is not None:
                return self.package(l)
            return self.runtime
        return self.package(self.__sc__.language)

    @property
    def runtime(self):
        if self.___hooked_configs___ is not None:
            if (l:=self.___hooked_configs___.get(self.___hooked_config_name___)) is not None:
                return self.package(l)
        return self.package(self.__sc__.language)

    def ml(self, code, language=None, replace=None, return_code=None):
        if language and code in self.package(language): return self.package(language).ml(code, replace, return_code)
        if code in self.runtime: return self.runtime.ml(code, replace, return_code)
        if code in self.default: return self.default.ml(code, replace, return_code)
        return replace or code if return_code or return_code is None else None

    def ml_name(self, language=None, replace=None, return_code=None): return self.ml(self.__s__.__s__.__i__.MLKEY_NAME, language, replace, return_code)
    def ml_abbreviation(self, language=None, replace=None, return_code=None): return self.ml(self.__s__.__i__.MLKEY_ABBR, language, replace, return_code)
    def ml_title(self, language=None, replace=None, return_code=None): return self.ml(self.__s__.__i__.MLKEY_TITLE, language, replace, return_code)
    def ml_description(self, language=None, replace=None, return_code=None): return self.ml(self.__s__.__i__.MLKEY_DESC, language, replace, return_code)
    def ml_link_title(self, language=None, replace=None, return_code=None): return self.ml(self.__s__.__i__.MLKEY_LINK_TITLE, language, replace, return_code)
    def ml_page_title(self, language=None, replace=None, return_code=None): return self.ml(self.__s__.__i__.MLKEY_PAGE_TITLE, language, replace, return_code)
    def ml_breadcrumb_title(self, language=None, replace=None, return_code=None): return self.ml(self.__s__.__i__.MLKEY_BDCD_TITLE, language, replace, return_code)



from ...nodes.tag import Tag


class Form(Tag):
    def __init__(self, u, action: str = None, attributes:dict[str, str|int|float|None]=None):
        if isinstance(attributes, dict):
            if action: attributes['action'] = action
        else:
            if action: attributes = {'action': action}
        Tag.__init__(self, u, None, attributes)
        if not self.attributes.get('method'): self.attributes['method'] = 'POST'



from ...types.he import He
from .configs import Configs
from .connection import Connection
from .tables import Tables

class Database(He):

    def __init__(self, u, host: str = None, user: str = None, password: str = None, dbname: str = None, port: int = None, prefix: str = None, connection: str = None, configs:dict=None): ...

    @property
    def configs(self)-> Configs: ...
    @property
    def tables(self)-> Tables: ...
    @property
    def connection(self)-> Connection: ...



from .......types.hd import Hd


class Arguments(Hd):
    @property
    def __websystem__(self): return self.__clients__.__websystem__
    @property
    def __clients__(self): return self.__client__.__clients__
    @property
    def __client__(self): return self.__request__.__client__
    @property
    def __request__(self): return self.__u__
    @property
    def __configs__(self): return self.__websystem__.configs

    def __getitem__(self, item): return super().__getitem__(item) if item in self else ''



from .....functions.decorators import parse_self_item_value, set_self_item_value_by_setter_with_agr
from ...configs import Configs as PluCfg


class Configs(PluCfg):
    def __init__(self, u):
        PluCfg.__init__(self, u, {
            'prefix': 'Dubeg',
            'printable': True,
            'saveable': False,
        })
    @property
    @parse_self_item_value()
    def prefix(self): return
    @prefix.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def prefix(self, value: str): pass
    @property
    @parse_self_item_value()
    def printable(self): return
    @printable.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def printable(self, value: str): pass
    @property
    @parse_self_item_value()
    def saveable(self): return
    @saveable.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def saveable(self, value: str): pass



from .......functions.decorators import parse_self_item_value_or_set_by_instance_he_by_module_name, \
    parse_self_item_value, parse_self_item_value_or_set_by_func
from ......types.hd import Hd


class Request(Hd):
    @property
    def __websystem__(self): return self.__clients__.__websystem__
    @property
    def __clients__(self): return self.__client__.__clients__
    @property
    def __client__(self): return self.__u__
    @property
    def __configs__(self): return self.__websystem__.configs

    @property
    @parse_self_item_value_or_set_by_instance_he_by_module_name()
    def headers(self): return

    @property
    @parse_self_item_value_or_set_by_func()
    def address(self): return self.__client__.address()

    @property
    @parse_self_item_value_or_set_by_instance_he_by_module_name()
    def arguments(self): return

    @property
    @parse_self_item_value_or_set_by_instance_he_by_module_name()
    def form(self): return

    @property
    @parse_self_item_value()
    def method(self): return
    def is_method_post(self): return self.method.upper() == 'POST'
    def is_method_get(self): return self.method.upper() == 'GET'

    @property
    @parse_self_item_value()
    def url(self): return

    @property
    @parse_self_item_value()
    def path(self): return

    @property
    @parse_self_item_value()
    def remote_addr(self): return



from .menus import Menus
from .menus.menu import Menu
from .menus.dropdown.core import Dropdown


class Navbar(dict):

    @property
    def logo(self)-> str: ...
    @logo.setter
    def logo(self, value): ...

    @property
    def title(self): ...
    @title.setter
    def title(self, value): ...

    @property
    def menus(self)-> Menus | list[Menu | Dropdown]: ...



from ...__data__ import Data


class Drop(Data):
    @property
    def conditions(self): return self.__conditions__
    def execute(self): return self.__queries__.delete_from_table(self.__table__.fullname, self.conditions)



from ..body import Body
from ..head import Head
from ... import Htmdoc
from ...nodes.tag import Tag
from ...nodes.wrap import Wrap


class Html(Tag):
    def __init__(self, u: Htmdoc | Wrap): ...

    @property
    def head(self)-> Head: ...
    @property
    def body(self)-> Body: ...


from ..website.webmod import Webmod
from ....types.he import He


class Webapp(He):
    def __init__(self, webmod: Webmod): ...

    @property
    def x(self)-> Webmod: ...

    def page_iterative(self): ...
    def page_content(self): ...

    def page_layout(self): ...

    def page_block_navbar(self): ...
    def page_block_breadcrumb(self): ...
    def page_block_navigations(self): ...
    def page_block_messages(self): ...
    def page_block_footer(self): ...
    def page_block_debugs(self): ...

    def page_block_1010(self): ...
    def page_block_1020(self): ...
    def page_block_1030(self): ...
    def page_block_1040(self): ...
    def page_block_1050(self): ...
    def page_block_1060(self): ...
    def page_block_1070(self): ...
    def page_block_1080(self): ...
    def page_block_1090(self): ...
    def page_block_1100(self): ...



from ...nodes.tag import Tag


class Th(Tag):
    def __init__(self, u, content=None, colspan:int=None, rowspan:int=None, attributes:dict[str, str|int|float|None]=None):
        Tag.__init__(self, u, None, content, attributes)
        if colspan: self.attributes.setdefault('colspan', colspan)
        if rowspan: self.attributes.setdefault('rowspan', rowspan)



from ..file import File
from ....functions.decorators import set_self_item_value_by_setter_with_agr, \
    parse_self_x_item_value_or_set_by_func, parse_dict_item_value_or_set_by_instance_he_by_module_name, \
    parse_self_x_item_value
from ...types.he import He


class CssMaker(He):
    def __init__(self, u, file: str | File = None, version=None):
        He.__init__(self, u)
        self.__x__['file'] = file if isinstance(file, File) else self.__sm__.file(file) if file else None
        self.__x__['version'] = version or '1.0.0'
    @property
    @parse_self_x_item_value()
    def file(self): return
    @property
    @parse_self_x_item_value()
    def version(self): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def sizes(self): return []
    @sizes.setter
    @set_self_item_value_by_setter_with_agr(value_type='lst')
    def sizes(self, value): return

    @property
    def cls_root(self): return self.cls(':root')
    @property
    def css_html(self): return self.cls('html')
    @property
    def css_body(self): return self.cls('body')

    @property
    @parse_self_x_item_value_or_set_by_func()
    def clses(self): return {}
    @parse_dict_item_value_or_set_by_instance_he_by_module_name(dict_attr_name='clses', dict_item_key_by_arg=0)
    def cls(self, name:str): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def medes(self): return {}
    @parse_dict_item_value_or_set_by_instance_he_by_module_name(dict_attr_name='clses', dict_item_key_by_func=True)
    def med(self, media_min_width:int=None, media_max_width:int=None, media_min_height:int=None, media_max_height:int=None):
        from .med import Med
        return Med.___make_name___(media_min_width, media_max_width, media_min_height, media_max_height)


    def save(self, minimized: bool = None):
        # fm = f.parent.child_file(f'min-{f.name}')
        rs = str(self)
        # rm = rs.replace('\n', '').replace('\t', '')
        if self.file:
            self.file.save_content(rs)
            if minimized:
                self.file.brother_file(f'{self.file.base_name}-min.css').save_content(rs.replace('\n', '').replace('\t', ''))
        else:
            print(rs)

    def __str__(self):
        ls = [
            f'/*!',
            f' * Hebill v{self.version}',
            f' * 版权所有 / Copyright 2026-2027 ©Hebill.',
            f' * 本作品仅限非商业用途使用，禁止商业行为。 / You may not use this work for commercial purposes.',
            f'*/',
        ]
        for n, c in self.medes.items():
            ls.append(c.__str__())
        for n, c in self.clses.items():
            ls.append(c.__str__())
        return '\n'.join(ls)



from .builtins import HeEro


def _dict(self, dict_attr_name, es):
    if dict_attr_name: d = getattr(self, dict_attr_name)
    else: d = self
    if not isinstance(d, dict): raise es.add(f'解析字典得到的属性（{type(d)}）没有继承 dict')
    return d
def _key(self, func, args, kwargs, dict_item_key, dict_item_key_by_arg, dict_item_key_by_func, func_name_striped, es):
    if isinstance(dict_item_key_by_arg, str):
        if dict_item_key_by_arg not in kwargs: raise es.add(f'解析字典键名 dict_item_key_by_arg[{dict_item_key_by_arg}] 未在 kwargs 设定')
        return kwargs[dict_item_key_by_arg]
    if isinstance(dict_item_key_by_arg, int):
        if dict_item_key_by_arg not in range(len(args)): raise es.add(f'解析字典键名 dict_item_key_by_arg[{dict_item_key_by_arg}] 超出了 len(args)[{len(args)}] 的范围')
        return args[dict_item_key_by_arg]
    if dict_item_key_by_func:
        return func(self, *args, **kwargs)
    if dict_item_key is not None:
        return dict_item_key
    if func_name_striped:
        return func.__name__.strip('_')
    return func.__name__
def parse_dict_item_value(dict_attr_name: str = None, dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None):
    def decorator(func):
        def wrapper(self, *args, **kwargs):
            es = HeEro(self, func, f'直接获取字典值', 'parse_dict_item_value', {'dict_attr_name': dict_attr_name, 'dict_item_key': dict_item_key, 'dict_item_key_by_arg': dict_item_key_by_arg, 'func_name_striped': func_name_striped})
            d, k = _dict(self, dict_attr_name, es), _key(self, func, args, kwargs, dict_item_key, dict_item_key_by_arg, False, func_name_striped, es)
            # print(type(self), dict_attr_name)
            if k in d: return d[k]
            return None
        return wrapper
    return decorator
def parse_self_item_value(dict_item_key: str | int = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None): return parse_dict_item_value(None, dict_item_key, dict_item_key_by_arg, func_name_striped)
def parse_self_x_item_value(dict_item_key: str | int = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None): return parse_dict_item_value('__x__', dict_item_key, dict_item_key_by_arg, func_name_striped)
def parse_self_d_item_value(dict_item_key: str | int = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None): return parse_dict_item_value('__d__', dict_item_key, dict_item_key_by_arg, func_name_striped)
def parse_dict_item_value_or_set_by_func(dict_attr_name: str = None, dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, execute_if_none: bool = None):
    def decorator(func):
        def wrapper(self, *args, **kwargs):
            es = HeEro(self, func, f'获取字典值，如果没有则运行函数赋值并返回', 'parse_dict_item_value_or_set_by_func', {'dict_attr_name': dict_attr_name, 'dict_item_key': dict_item_key, 'dict_item_key_by_arg': dict_item_key_by_arg, 'execute_if_none': func_name_striped, 'func_name_striped': execute_if_none})
            d, k = _dict(self, dict_attr_name, es), _key(self, func, args, kwargs, dict_item_key, dict_item_key_by_arg, False, func_name_striped, es)
            if k not in d or (d[k] is None and execute_if_none): d[k] = func(self, *args, **kwargs)
            return d[k]
        return wrapper
    return decorator
def parse_self_item_value_or_set_by_func(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, execute_if_none: bool = None): return parse_dict_item_value_or_set_by_func(None, dict_item_key, dict_item_key_by_arg, func_name_striped, execute_if_none)
def parse_self_x_item_value_or_set_by_func(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, execute_if_none: bool = None): return parse_dict_item_value_or_set_by_func('__x__', dict_item_key, dict_item_key_by_arg, func_name_striped, execute_if_none)
def parse_self_d_item_value_or_set_by_func(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, execute_if_none: bool = None): return parse_dict_item_value_or_set_by_func('__d__', dict_item_key, dict_item_key_by_arg, func_name_striped, execute_if_none)

def _module_name(func, module_name, module_name_striped):
    if module_name_striped:
        module_name_by_func = func.__name__.strip('_')
    else:
        module_name_by_func = func.__name__
    if module_name is None:
        return module_name_by_func
    if module_name == '#':
        return module_name_by_func
    if module_name.endswith('.#'):
        return f'{module_name[:-1]}{module_name_by_func}' # ..#
    if module_name.endswith('#'):
        return f'{module_name[:-1]}.{module_name_by_func}' # .abc#
    return module_name

def _instance(self, func, args, kwargs, module_name, arg0, module_name_striped, search_from_self, es):
    from ..functions.builtins import join_mod_path, parse_mod_path_by_pycls, instance_pyobj_by_mod_path
    from ..system.types.he import He
    ags = [self if arg0 is None else getattr(self, arg0), *args]
    module_name = _module_name(func, module_name, module_name_striped)
    if module_name.startswith('/'): # 绝对路径模块
        return instance_pyobj_by_mod_path(module_name, ags, kwargs)
    # 相对路径模块 ！需要从定义函数的类开始偏移！
    if module_name.startswith('.'):
        offset = len(module_name) - len(mod_r := module_name.lstrip('.'))
        for cls in self.__class__.__mro__:
            if issubclass(cls, He) and cls.__dict__.get(func.__name__, None):
                clsmod = parse_mod_path_by_pycls(cls)
                target_module_name = join_mod_path(*clsmod.split('.')[:-offset], mod_r)
                return instance_pyobj_by_mod_path(target_module_name, ags, kwargs)
        raise es.add(f'实例函数：[{type(self)}].[{func}]() 函数解析实例化相对模块名称 {module_name} 没有找定义函数的主类！')
    if '.' in module_name: # 直属多级子模块 ！！！只需要从定义函数的类查询多级子模块！！！
        for cls in self.__class__.__mro__:
            if issubclass(cls, He) and cls.__dict__.get(func.__name__, None):
                target_module_name = parse_mod_path_by_pycls(cls, module_name)
                return instance_pyobj_by_mod_path(target_module_name, ags, kwargs)
        raise es.add(f'多级子模块名称 {module_name} 没有找定义函数的主类！')
    # 不从 self 开始直接实例化定义函数的类的相对模块
    if search_from_self is False:
        for cls in self.__class__.__mro__:
            if issubclass(cls, He) and cls.__dict__.get(func.__name__, None):
                target_module_name = parse_mod_path_by_pycls(cls, module_name)
                try: return instance_pyobj_by_mod_path(target_module_name, ags, kwargs)
                except Exception as e: raise es.add(f'不从 self 开始直接实例化函数所在类的相对模块： {target_module_name} 失败，错误信息：{e}')
        raise es.add(f'直属子模块名称 {module_name} 没有找定义函数的主类！')
    # 从 self 开始搜索尝试实例化类的相对模块直到函数所在类
    for cls in self.__class__.__mro__:
        if issubclass(cls, He):
            target_module_name = parse_mod_path_by_pycls(cls, module_name)
            # 排查部分：-------------------------------------------------------------------------------  需要删除
            # if target_module_name == 'hebill.builtin.system.components.web_system':
            #     return instance_pyobj_by_mod_path(target_module_name, ags, kwargs)
            # ----------------------------------------------------------------------------------------------------
            try: return instance_pyobj_by_mod_path(target_module_name, ags, kwargs)
            except Exception as e:
                es.add(f'循环尝试实例化模块： {target_module_name} 未成功')
                es.add(e)
        if cls.__dict__.get(func.__name__, None): break
    raise es.add(f'直属子模块名称 {module_name} 没有找定义函数的主类！')

def instance_he_by_module_name(module_name: str = None, arg0: str = None, module_name_striped: bool = None, search_from_self: bool = None):
    def decorator(func):
        def wrapper(self, *args, **kwargs):
            es = HeEro(self, func, f'实例化模块赋值并返回', 'instance_he_by_module_name', {'module_name': module_name, 'arg0': arg0, 'module_name_striped': module_name_striped, 'search_from_self': search_from_self})
            return _instance(self, func, args, kwargs, module_name, arg0, module_name_striped, search_from_self, es)
        return wrapper
    return decorator

def parse_dict_item_value_or_set_by_instance_he_by_module_name(dict_attr_name: str = None, dict_item_key: str = None, dict_item_key_by_arg: str | int = None, dict_item_key_by_func: bool = None, func_name_striped: bool = None, execute_if_none: bool = None, module_name: str = None, arg0: str = None, module_name_striped: bool = None, search_from_self: bool = None):
    def decorator(func):
        def wrapper(self, *args, **kwargs):
            es = HeEro(self, func, f'获取字典值，如果没有则运行函数赋值并返回', 'parse_dict_item_value_or_set_by_instance_he_by_module_name', {'dict_attr_name': dict_attr_name, 'dict_item_key': dict_item_key, 'dict_item_key_by_arg': dict_item_key_by_arg, 'func_name_striped': execute_if_none, 'execute_if_none': module_name, 'module_name': func_name_striped, 'arg0': arg0, 'module_name_striped': module_name_striped, 'search_from_self': search_from_self})
            d, k = _dict(self, dict_attr_name, es), _key(self, func, args, kwargs, dict_item_key, dict_item_key_by_arg, dict_item_key_by_func, func_name_striped, es)
            if k not in d or (d[k] is None and execute_if_none): d[k] = _instance(self, func, args, kwargs, module_name, arg0, module_name_striped, search_from_self, es)
            return d[k]
        return wrapper
    return decorator
def parse_self_item_value_or_set_by_instance_he_by_module_name(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, dict_item_key_by_func: bool = None, func_name_striped: bool = None, execute_if_none: bool = None, module_name: str = None, arg0: str = None, module_name_striped: bool = None, search_from_self: bool = None):
    return parse_dict_item_value_or_set_by_instance_he_by_module_name(None, dict_item_key, dict_item_key_by_arg, dict_item_key_by_func, func_name_striped, execute_if_none, module_name, arg0, module_name_striped, search_from_self)
def parse_self_x_item_value_or_set_by_instance_he_by_module_name(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, dict_item_key_by_func: bool = None, func_name_striped: bool = None, execute_if_none: bool = None, module_name: str = None, arg0: str = None, module_name_striped: bool = None, search_from_self: bool = None):
    return parse_dict_item_value_or_set_by_instance_he_by_module_name('__x__', dict_item_key, dict_item_key_by_arg, dict_item_key_by_func, func_name_striped, execute_if_none, module_name, arg0, module_name_striped, search_from_self)
def parse_self_d_item_value_or_set_by_instance_he_by_module_name(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, dict_item_key_by_func: bool = None, func_name_striped: bool = None, execute_if_none: bool = None, module_name: str = None, arg0: str = None, module_name_striped: bool = None, search_from_self: bool = None):
    return parse_dict_item_value_or_set_by_instance_he_by_module_name('__d__', dict_item_key, dict_item_key_by_arg, dict_item_key_by_func, func_name_striped, execute_if_none, module_name, arg0, module_name_striped, search_from_self)

def _set_dict_item_value_by_setter_with_agr(self, func, args, kwargs, dict_attr_name, dict_item_key, dict_item_key_by_arg, func_name_striped, value_agr_sn, value_type, es):
    d, k, val, = _dict(self, dict_attr_name, es), _key(self, func, args, kwargs, dict_item_key, dict_item_key_by_arg, False, func_name_striped, es), args[value_agr_sn or 0]
    if isinstance(value_type, str): vts = [value_type.lower()]
    elif isinstance(value_type, (list, tuple)): vts = [i.lower() for i in value_type]
    else: vts = []
    if not vts: d[k] = val; return
    def _float(v):
        if isinstance(v, (int, float)): return float(v)
        if isinstance(v, str):
            try: return float(v.replace(',', '').replace(' ', ''))
            except ValueError: return None
        return None
    def _is_scalar(v):
        return isinstance(v, (str, int, float))
    def _to_int(v, *, cond=lambda x: True):
        fv = _float(v)
        if fv is None or fv != int(fv) or not cond(fv): return None
        return int(fv)
    converters = {
        # 基础
        'bol': lambda v: bool(v),
        'bool': lambda v: bool(v),
        'non': lambda v: None,
        'none': lambda v: None,

        # 字符串
        'str': lambda v: str(v) if _is_scalar(v) else None,
        'string': lambda v: str(v) if _is_scalar(v) else None,
        'strue': lambda v: str(v) if _is_scalar(v) and str(v) else None,
        'string_unempty': lambda v: str(v) if _is_scalar(v) and str(v) else None,

        # 字典
        'dit': lambda v: dict(v) if isinstance(v, dict) else None,
        'dict': lambda v: dict(v) if isinstance(v, dict) else None,
        # 列表
        'lst': lambda v: list(v) if isinstance(v, list) else None,
        'list': lambda v: list(v) if isinstance(v, list) else None,

        # 浮点
        'flt': _float,
        'float': _float,
        'fltp': lambda v: f if (f := _float(v)) is not None and f > 0 else None,
        'float_positive': lambda v: f if (f := _float(v)) is not None and f > 0 else None,
        'fltn': lambda v: f if (f := _float(v)) is not None and f < 0 else None,
        'float_negtive': lambda v: f if (f := _float(v)) is not None and f < 0 else None,
        'fltup': lambda v: f if (f := _float(v)) is not None and f <= 0 else None,
        'float_unpositive': lambda v: f if (f := _float(v)) is not None and f <= 0 else None,
        'fltun': lambda v: f if (f := _float(v)) is not None and f >= 0 else None,
        'float_unnegtive': lambda v: f if (f := _float(v)) is not None and f >= 0 else None,
        # 整数
        'int': lambda v: _to_int(v),
        'integer': lambda v: _to_int(v),
        'intp': lambda v: _to_int(v, cond=lambda x: x > 0),
        'integer_positive': lambda v: _to_int(v, cond=lambda x: x > 0),
        'intn': lambda v: _to_int(v, cond=lambda x: x < 0),
        'integer_negtive': lambda v: _to_int(v, cond=lambda x: x < 0),
        'intup': lambda v: _to_int(v, cond=lambda x: x <= 0),
        'integer_unpositive': lambda v: _to_int(v, cond=lambda x: x <= 0),
        'intun': lambda v: _to_int(v, cond=lambda x: x >= 0),
        'integer_unnegtive': lambda v: _to_int(v, cond=lambda x: x >= 0),
        # 语言
        'lan': lambda v: v if v in self.__s__.__i__.LANS else None,
        'language': lambda v: v if v in self.__s__.__i__.LANS else None,
        # 网站请求模式
        'webappmode': lambda v: v if v in self.__s__.__i__.WEBMODS else None,
    }
    for vt in vts:
        conv = converters.get(vt)
        if not conv: raise es.add(f'不支持提供的赋值类型{vts}')
        result = conv(val)
        if result is not None:
            d[k] = result
            return
    return

def set_dict_item_value_by_setter_with_agr(dict_attr_name: str = None, dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, value_agr_sn:int=None, value_type:str=None):
    def decorator(func):
        def wrapper(self, *args, **kwargs):
            es = HeEro(self, func, f'设置字典值，从 args 中获取对应值赋值', 'set_dict_item_value_by_setter_with_agr', {'dict_attr_name': dict_attr_name, 'dict_item_key': dict_item_key, 'dict_item_key_by_arg': dict_item_key_by_arg, 'func_name_striped': func_name_striped, 'value_agr_sn': value_agr_sn, 'value_type': value_type})
            _set_dict_item_value_by_setter_with_agr(self, func, args, kwargs, dict_attr_name, dict_item_key, dict_item_key_by_arg, func_name_striped, value_agr_sn, value_type, es)
            return None
        return wrapper
    return decorator
def set_self_item_value_by_setter_with_agr(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, value_agr_sn:int=None, value_type:str=None): return set_dict_item_value_by_setter_with_agr(None, dict_item_key, dict_item_key_by_arg, func_name_striped, value_agr_sn, value_type)
def set_self_x_item_value_by_setter_with_agr(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, value_agr_sn:int=None, value_type:str=None): return set_dict_item_value_by_setter_with_agr('__x__', dict_item_key, dict_item_key_by_arg, func_name_striped, value_agr_sn, value_type)
def set_self_d_item_value_by_setter_with_agr(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, value_agr_sn:int=None, value_type:str=None): return set_dict_item_value_by_setter_with_agr('__d__', dict_item_key, dict_item_key_by_arg, func_name_striped, value_agr_sn, value_type)

def _redirect_to_attribute_function(self, func, args, kwargs, func_attr_name, func_name_striped, es):
    ns = func_attr_name.split('.')
    if ns[-1].endswith('#'):
        ns[-1] = ns[-1].strip('#')
        ns.append(func.__name__.strip('_') if func_name_striped else func.__name__)
    def loop(o, i):
        if not hasattr(o, ns[i]):
            raise es.add(f'self.{'.'.join(ns[:i])} 函数属性不存在！')
        r = getattr(o, ns[i])
        if i == len(ns) - 1: return r
        return loop(r, i + 1)
    f = loop(self, 0)
    if callable(f):
        try: return f(*args, **kwargs)
        except Exception as e: return es.add(f'self.{'.'.join(ns)} 属性函数运行失败: {e}！')
    raise es.add(f'self.{'.'.join(ns)} 属性不不是函数！')
def redirect_to_attribute_function(func_attr_name: str, func_name_striped: bool = None):
    def decorator(func):
        def wrapper(self, *args, **kwargs):
            es = HeEro(self, func, f'函数转发', 'redirect_to_attribute_function', {'func_attr_name': func_attr_name, 'func_name_striped': func_name_striped})
            return _redirect_to_attribute_function(self, func, args, kwargs, func_attr_name, func_name_striped, es)
        return wrapper
    return decorator

def _redirect_to_attribute_property(self, func, func_attr_name, func_name_striped, es):
    ns = func_attr_name.split('.')
    if ns[-1].endswith('#'):
        ns[-1] = ns[-1].strip('#')
        ns.append(func.__name__.strip('_') if func_name_striped else func.__name__)
    def loop(o, i):
        if not hasattr(o, ns[i]):
            raise es.add(f'self.{'.'.join(ns[:i])} 属性不存在！')
        r = getattr(o, ns[i])
        if i == len(ns) - 1: return r
        return loop(r, i + 1)
    return loop(self, 0)
def redirect_to_attribute_property(func_attr_name: str, func_name_striped: bool = None):
    def decorator(func):
        def wrapper(self):
            es = HeEro(self, func, f'属性转发', 'redirect_to_attribute_property', {'func_attr_name': func_attr_name, 'func_name_striped': func_name_striped})
            return _redirect_to_attribute_property(self, func, func_attr_name, func_name_striped, es)
        return wrapper
    return decorator



import os
import subprocess
from ..folder import Folder
from ..mod_type import ModType
from ...types.he import He
from ....functions.builtins import HeEro
from ....functions.decorators import parse_self_x_item_value_or_set_by_instance_he_by_module_name


class ExePacker(He):
    def __init__(self, u, mainpy:str, website_module:str=None, one_file:bool=False, one_dir:bool=False, show_console:bool=False, pack_overwrite:bool=False):
        super().__init__(u)
        exename = os.path.splitext(os.path.basename(mainpy))[0]
        if exename.endswith('main.py'): exename = exename.replace('main.py', '.py')
        self.configs.update({
            'mainpy': mainpy,
            'website_module': website_module,
            'exe_name': (exename:=os.path.splitext(os.path.basename(mainpy))[0]),
            'one_file': one_file,
            'one_dir': one_dir,
            'show_console': show_console,
            'pack_overwrite': pack_overwrite,
            'root_folders': [self.__s__.__i__.hebill_folder],
            'root_modules': [self.__s__.__i__.hebill_mod_type],
            'dist_path': (pkgdir:=self.__s__.__i__.project_folder.child_folder('.packages').child_folder(exename)).child_folder('dist'),
            'build_path': pkgdir.child_folder(exename).child_folder('build'),
            'spec_path': pkgdir.child_folder(exename),
            'comd_path': pkgdir.child_folder(exename),
        })

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def configs(self): return

    def add_root_folder(self, folder):
        if folder:
            if not isinstance(folder, Folder): folder = self.__s__.components.folder(folder)
            self.configs.root_folders.append(folder)
    def ___check_root_module___(self, module):
        if module:
            ero = HeEro(ExePacker, '___check_root_module___', '检验根模块')
            if not isinstance(module, ModType):
                try: return self.__s__.components.mod_path(module).mod_type
                except Exception as e: raise (ero.add(f'无法导入根模块：{module}', e))
        return None

    def add_root_module(self, module):
        if module:=self.___check_root_module___(module):
            self.configs.root_folders.append(module.folder)
            self.configs.root_modules.append(module)

    def __output_commands__(self) -> list[str]:
        website_module = self.___check_root_module___(self.configs.website_module)
        cmd = ['PyInstaller', '--clean']
        # --name 必须拆开
        cmd.extend(['--name', self.configs.exe_name])
        if self.configs.one_dir: cmd.append('--onedir')
        elif self.configs.one_file: cmd.append('--onefile')
        if not self.configs.show_console: cmd.append('--windowed')
        if self.configs.pack_overwrite: cmd.append('--noconfirm')
        if self.configs.dist_path: cmd.extend(['--distpath', self.configs.dist_path])
        if self.configs.build_path: cmd.extend(['--workpath', self.configs.build_path])
        if self.configs.spec_path: cmd.extend(['--specpath', self.configs.spec_path])
        ico_added = False
        if self.configs.icon_file:
            ico_added = True
            cmd.extend(['--icon', self.configs.icon_file])
        if not ico_added:
            if website_module:
                ico = website_module.folder.child_file('logo.ico')
                if ico.is_exists():
                    ico_added = True
                    cmd.extend(['--icon', ico])
        if not ico_added:
            ico = self.__s__.__i__.hebill_folder.child_file('logo.ico')
            if ico.is_exists():
                cmd.extend(['--icon', ico])

        if self.configs.version_file: cmd.extend(['--version-file', self.configs.version_file])
        for pkg in self.configs.root_modules: cmd.extend(['--collect-submodules', pkg])
        if website_module: cmd.extend(['--collect-submodules', website_module])
        '''def collect(fd: Folder):
            if fd.is_exists():
                cmd.extend(['--add-data', f'{fd};{os.path.basename(fd)}'])
                """for dirpath, _, filenames in os.walk(fd):
                    for file in filenames:
                        if file.endswith(('.py', '.pyc', '.pyi')):continue
                        src = os.path.join(dirpath, file)
                        dst = os.path.relpath(src, self.__s__.__i__.project_folder)
                        # --add-data 也必须拆开
                        cmd.extend(['--add-data', f'{src};{dst}'])"""
        for folder in self.configs.root_folders: collect(folder)
        if website_module: collect(website_module.folder)'''

        cmd.append(self.configs.mainpy)
        return cmd

    def pack(self):
        if self.configs.comd_path:
            cmd = self.__output_commands__()
            lines = [cmd[0] + ' ^']
            for arg in cmd[1:-1]: lines.append(f'  {arg} ^')
            lines.append(f'  {cmd[-1]}')
            self.configs.comd_path.child_file(f'{self.configs.exe_name}.cmd').save_content('\n'.join(lines))
        subprocess.run(self.__output_commands__(), check=True)

    def output_command(self) -> str: # 仅用于“显示”，不是执行
        return ' '.join(f'"{c}"' if ' ' in c else c for c in self.__output_commands__())

    def save_command(self, file=None):
        if not file: file = self.configs.exe_name + '.cmd'
        if isinstance(file, str): file = self.__sm__.file(file)
        cmd = self.__output_commands__()
        lines = [cmd[0] + ' ^']
        for arg in cmd[1:-1]: lines.append(f'  {arg} ^')
        lines.append(f'  {cmd[-1]}')
        file.save_content('\n'.join(lines))



class Menu(dict):
    def __init__(self, title=None, url=None):
        super().__init__()
        self.title = title
        self.url = url

    @property
    def title(self): return self.get('title')
    @title.setter
    def title(self, value): self['title'] = value

    @property
    def url(self): return self.get('url')
    @url.setter
    def url(self, value): self['url'] = value

    def is_menu(self): return True
    def is_dropdown(self): return False



from .builtins import HeEro
from ..system.types.he import He



def _dict(self:He, dict_attr_name, es:HeEro): ...
def _key(func, args, kwargs, dict_item_key, dict_item_key_by_arg, dict_item_key_by_func, func_name_striped, es:HeEro): ...

def parse_dict_item_value(dict_attr_name: str = None, dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None): ...
def parse_self_item_value(dict_item_key: str | int = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None): ...
def parse_self_x_item_value(dict_item_key: str | int = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None): ...
def parse_self_d_item_value(dict_item_key: str | int = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None): ...
def parse_dict_item_value_or_set_by_func(dict_attr_name: str = None, dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, execute_if_none: bool = None): ...
def parse_self_item_value_or_set_by_func(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, execute_if_none: bool = None): ...
def parse_self_x_item_value_or_set_by_func(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, execute_if_none: bool = None): ...
def parse_self_d_item_value_or_set_by_func(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, execute_if_none: bool = None): ...

def _module_name(func, module_name, module_name_striped:bool|None): ...
def _instance(self:He, func, args, kwargs, module_name, arg0, module_name_striped: bool|None, search_from_self: bool|None, es:HeEro): ...
def instance_he_by_module_name(module_name: str = None, arg0: str = None, module_name_striped: bool = None, search_from_self: bool = None): ...
def parse_dict_item_value_or_set_by_instance_he_by_module_name(dict_attr_name: str = None, dict_item_key: str = None, dict_item_key_by_arg: str | int = None, dict_item_key_by_func: bool = None, func_name_striped: bool = None, execute_if_none: bool = None, module_name: str = None, arg0: str = None, module_name_striped: bool = None, search_from_self: bool = None): ...
def parse_self_item_value_or_set_by_instance_he_by_module_name(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, dict_item_key_by_func: bool = None, func_name_striped: bool = None, execute_if_none: bool = None, module_name: str = None, arg0: str = None, module_name_striped: bool = None, search_from_self: bool = None): ...
def parse_self_x_item_value_or_set_by_instance_he_by_module_name(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, dict_item_key_by_func: bool = None, func_name_striped: bool = None, execute_if_none: bool = None, module_name: str = None, arg0: str = None, module_name_striped: bool = None, search_from_self: bool = None): ...
def parse_self_d_item_value_or_set_by_instance_he_by_module_name(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, dict_item_key_by_func: bool = None, func_name_striped: bool = None, execute_if_none: bool = None, module_name: str = None, arg0: str = None, module_name_striped: bool = None, search_from_self: bool = None): ...

def _set_dict_item_value_by_setter_with_agr(self:He, func, args, kwargs, dict_attr_name, dict_item_key, dict_item_key_by_arg, func_name_striped, value_agr_sn, value_type, es:HeEro): ...
def set_dict_item_value_by_setter_with_agr(dict_attr_name: str = None, dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, value_agr_sn:int=None, value_type:str|list[str]|tuple=None): ...
def set_self_item_value_by_setter_with_agr(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, value_agr_sn:int=None, value_type:str|list[str]|tuple=None): ...
def set_self_x_item_value_by_setter_with_agr(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, value_agr_sn:int=None, value_type:str|list[str]|tuple=None): ...
def set_self_d_item_value_by_setter_with_agr(dict_item_key: str = None, dict_item_key_by_arg: str | int = None, func_name_striped: bool = None, value_agr_sn:int=None, value_type:str|list[str]|tuple=None): ...

def _redirect_to_attribute_function(self:He, func, args, kwargs, func_attr_name, func_name_striped, es:HeEro): ...
def redirect_to_attribute_function(func_attr_name: str = None, func_name_striped: bool = None): ...

def _redirect_to_attribute_property(self:He, func, func_attr_name, func_name_striped, es:HeEro): ...
def redirect_to_attribute_property(func_attr_name: str = None, func_name_striped: bool = None): ...



from ....functions.decorators import parse_self_x_item_value, parse_self_x_item_value_or_set_by_func, \
    parse_self_x_item_value_or_set_by_instance_he_by_module_name


class He:
    def __init__(self, u):
        self.__x = {}
        from ....system import System
        if isinstance(u, System):
            self.__x__['__s__'] = u
            self.__x__['__u__'] = None
        elif isinstance(u, He):
            self.__x__['__s__'] = u.__s__
            self.__x__['__u__'] = u
        else:
            raise TypeError(f'{type(u)} 不是 System | He。')

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name(module_name_striped=True)
    def __i__(self): return
    @property
    def __id__(self): return id(self)
    @property
    def __x__(self): return self.__x

    @property
    @parse_self_x_item_value()
    def __s__(self): return
    @property
    def __sm__(self): return self.__s__.components
    @property
    def __sc__(self): return self.__s__.configs
    @property
    def __sd__(self): return self.__s__.debugs
    @property
    def __st__(self): return self.__s__.types

    @property
    @parse_self_x_item_value()
    def __u__(self): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def __l__(self): return []

    @property
    @parse_self_x_item_value_or_set_by_func()
    def __d__(self): return {}

    @property
    @parse_self_x_item_value_or_set_by_func()
    def __t__(self): return ''



from .....configs import Configs
from .. import Webmod


class Settings(Configs):
    def __init__(self, u: Webmod): ...
    @property
    def __u__(self)-> Webmod: ...

    @property
    def page_feature(self)-> str: ...
    @page_feature.setter
    def page_feature(self, value) -> None: ...
    def is_page_feature_general(self)-> bool: ...
    def is_page_feature_sign_in(self)-> bool: ...
    def is_page_feature_sign_out(self)-> bool: ...
    def is_page_feature_sign_reset(self)-> bool: ...
    def is_page_feature_sign_up(self)-> bool: ...
    @property
    def power(self)-> str: ...
    @power.setter
    def power(self, value) -> None: ...
    def is_power_public(self)-> bool: ...
    def is_power_groups(self)-> bool: ...
    def is_power_inherited(self)-> bool: ...
    def is_power_members(self)-> bool: ...

    @property
    def page_layout(self)-> str: ...
    @page_layout.setter
    def page_layout(self, value) -> None: ...

    @property
    def icon(self)-> str: ...
    @icon.setter
    def icon(self, value) -> None: ...



from ...types.he import He
from ....functions.builtins import HeEro
from ....functions.decorators import parse_self_x_item_value_or_set_by_func, \
    parse_self_x_item_value_or_set_by_instance_he_by_module_name


class WebSystem(He):
    def __init__(self, u, module: str, host: str=None, port: int=None, db_host: str=None, db_user: str=None, db_password: str=None, db_dbname: str=None, db_port: int=None, db_prefix: str=None, configs:dict=None):
        ero = HeEro(WebSystem, '__init__', '网站系统实例化')
        super().__init__(u)
        self.configs.update(configs)
        """设置参数"""
        self.configs.module = module
        self.configs.host = host
        self.configs.port = port
        self.configs.mysql_host = db_host
        self.configs.mysql_user = db_user
        self.configs.mysql_password = db_password
        self.configs.mysql_dbname = db_dbname
        self.configs.mysql_port = db_port
        self.configs.mysql_prefix = db_prefix
        # 由于在 Config.__init__ 调用时候 'module', 'host', 'port' 还未设置，会造成死循环，所以在这里更新网站系统设置
        if not self.is_website_exists():
            raise ero.add(f'网站系统设置的网站根模块 {self.configs.module} 不存在')
        if self.website_mod_type.data_configs_file.is_exists():
            if isinstance(data:=self.website_mod_type.data_configs_file.parse_json(), dict):
                for k, v in data.items():
                    if k not in ['module', 'host', 'port']:
                        self.configs.set(k, v)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def __debugs_prefix__(self): return [self.configs.module]

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def configs(self): return

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def database(self): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def server(self): from .servers.flask_server import FlaskServer; return FlaskServer(self)

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def clients(self): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def website_mod_path(self): return self.__sm__.mod_path(self.configs.module)
    @property
    def website_mod_type(self): return self.website_mod_path.mod_type
    def is_website_exists(self): return self.website_mod_type is not None
    @property
    def website_folder(self): return self.website_mod_type.folder if self.is_website_exists() else None



from ....types.he import He
from .. import Database
from .table import Table
from ..connection import Connection
from ..configs import Configs


class Tables(He):
    @property
    def __database__(self)-> Database: ...
    @property
    def __connection__(self)-> Connection: ...
    @property
    def __configs__(self)-> Configs: ...
    def table(self, name)-> Table: ...
    def search_names(self, keyword=None, startwith: bool = None, endwith: bool = None) -> list[str]: ...



from .......functions.builtins import compress, join_url
from .......functions.decorators import parse_self_item_value_or_set_by_func, set_self_item_value_by_setter_with_agr, parse_self_item_value
from .....configs import Configs


class Address(Configs):
    def __init__(self, client, mode: str = None, module: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None):
        Configs.__init__(self, client)
        self.mode = mode
        self.module = module
        self.arguments = arguments
        self.keyword = keyword
        self.page = page
        self.orders = orders
        self.filters = filters
        self.items_per_page = items_per_page
        self.pages_per_group = pages_per_group
        self.reference = reference
        self.language = language
        self.security = security
        self.___defaults___.update(self)

    @property
    def __client__(self): return self.__u__
    @property
    def ___websystem_configs___(self): return self.__client__.__websystem__.configs

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def language(self): return self.__client__.request.address.language
    @language.setter
    @set_self_item_value_by_setter_with_agr(value_type='language')
    def language(self, value): pass
    def is_language_chinese(self): return self.language == 'cn'
    def is_language_english(self): return self.language == 'gb'
    def set_language_chinese(self): self.language = 'cn'
    def set_language_english(self): self.language = 'gb'

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def mode(self): return 'page'
    @mode.setter
    @set_self_item_value_by_setter_with_agr(value_type='webappmode')
    def mode(self, value): pass
    def is_mode_page(self): return self.mode == 'page'
    def is_mode_json(self): return self.mode == 'json'
    def is_mode_ajax(self): return self.mode == 'ajax'
    def is_mode_grid(self): return self.mode == 'grid'
    def is_mode_file(self): return self.mode == 'file'
    def is_mode_not_page(self): return not self.is_mode_page()
    def is_mode_not_json(self): return not self.is_mode_json()
    def is_mode_not_ajax(self): return not self.is_mode_ajax()
    def is_mode_not_grid(self): return not self.is_mode_grid()
    def is_mode_not_file(self): return not self.is_mode_file()

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def module(self): return ''
    @module.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def module(self, value): pass

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def security(self): return ''
    @security.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def security(self, value): pass

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def arguments(self): return {}
    @arguments.setter
    @set_self_item_value_by_setter_with_agr(value_type='dit')
    def arguments(self, value): pass

    @property
    @parse_self_item_value()
    def keyword(self): return
    @keyword.setter
    @set_self_item_value_by_setter_with_agr(value_type=('str', 'non'))
    def keyword(self, value):pass

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def page(self): return 1
    @page.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def page(self, value): pass

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def orders(self): return {}
    @orders.setter
    @set_self_item_value_by_setter_with_agr(value_type='dit')
    def orders(self, value): pass

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def filters(self): return {}
    @filters.setter
    @set_self_item_value_by_setter_with_agr(value_type='dit')
    def filters(self, value): pass

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def items_per_page(self): return self.___websystem_configs___.pagination_items_per_page
    @items_per_page.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def items_per_page(self, value): pass

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def pages_per_group(self): return self.___websystem_configs___.pagination_pages_per_group
    @pages_per_group.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def pages_per_group(self, value):  pass

    @property
    @parse_self_item_value_or_set_by_func(execute_if_none=True)
    def reference(self): return ''
    @reference.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def reference(self, value: str): pass

    def copy(self):
        address = Address(self.__client__)
        address.update(self.data())
        return address

    def output(self):
        cs = self.___websystem_configs___
        data = self.data()
        url = [self.___websystem_configs___.url_root_path.strip('/'), data[cs.url_key_mode]]
        data.__delitem__(cs.url_key_mode)
        url.append(compress(self.data()))
        return join_url(*url)
        if self.___websystem_configs___.url_root_path.strip('/'):
            return f'/{self.___websystem_configs___.url_root_path.strip('/')}/{url}'
        return f'/{url}'
    def update(self, m: dict, /, **kwargs):
        for k, v in m.items():
            match k:
                case 'mode' | self.___websystem_configs___.url_key_mode: self.mode = v
                case 'module' | self.___websystem_configs___.url_key_module: self.module = v
                case 'arguments' | self.___websystem_configs___.url_key_arguments: self.arguments = v
                case 'keyword' | self.___websystem_configs___.url_key_keyword: self.keyword = v
                case 'page' | self.___websystem_configs___.url_key_page: self.page = v
                case 'orders' | self.___websystem_configs___.url_key_orders: self.orders = v
                case 'filters' | self.___websystem_configs___.url_key_filters: self.filters = v
                case 'items_per_page' | self.___websystem_configs___.url_key_items_per_page: self.items_per_page = v
                case 'pages_per_group' | self.___websystem_configs___.url_key_pages_per_group: self.pages_per_group = v
                case 'reference' | self.___websystem_configs___.url_key_reference: self.reference = v
                case 'language' | self.___websystem_configs___.url_key_language: self.language = v
                case 'security' | self.___websystem_configs___.url_key_security: self.security = v

    def data(self):
        cs = self.___websystem_configs___
        data: dict = {
            cs.url_key_mode: self.mode,
            cs.url_key_language: self.language,
            cs.url_key_module: self.module
        }
        if self.arguments:
            # data[self.___cr___.url_key_arguments] = json.dumps(self.arguments)
            data[cs.url_key_arguments] = self.arguments
        if self.keyword is not None:
            data[cs.url_key_keyword] = self.keyword
            data[cs.url_key_page] = self.page
            if self.orders:
                data[cs.url_key_orders] = self.orders
            else:
                data[cs.url_key_orders] = {}
            if self.filters:
                data[cs.url_key_filters] = self.filters
            else:
                data[cs.url_key_filters] = {}
            if self.items_per_page:
                data[cs.url_key_items_per_page] = self.items_per_page
            else:
                data[cs.url_key_items_per_page] = cs.pagination_items_per_page
            if self.pages_per_group:
                data[cs.url_key_pages_per_group] = self.pages_per_group
            else:
                data[cs.url_key_pages_per_group] = cs.pagination_pages_per_group
        if self.reference:
            data[cs.url_key_reference] = self.reference
        if self.security:
            data[cs.url_key_security] = self.security
        return data

    def redirect(self): return self.__client__.__websystem__.server.make_response_redirect(self.output())

    # def __str__(self): return self.data().__str__()


from ...database import Database as PluginDatabase


class Database(PluginDatabase):
    def __init__(self, system):
        super().__init__(system)
        self.configs.hook('mysql_host', system.configs, 'host')
        self.configs.hook('mysql_user', system.configs, 'user')
        self.configs.hook('mysql_password', system.configs, 'password')
        self.configs.hook('mysql_dbname', system.configs, 'dbname')
        self.configs.hook('mysql_port', system.configs, 'port')
        self.configs.hook('mysql_prefix', system.configs, 'prefi')



from types import ModuleType

from ....functions.builtins import parse_mod_path_by_pyobj, parse_mod_path_by_pycls, import_pycls_by_mod_path, parse_dirpath_by_pymod
from ....functions.decorators import parse_self_x_item_value_or_set_by_func, parse_self_x_item_value
from ...types.he import He
from ...types.hs import Hs



class ModType(Hs):
    def __new__(cls, u, pymod: ModuleType | type | He):
        if isinstance(pymod, He):
            obj = Hs.__new__(cls, u, parse_mod_path_by_pyobj(pymod))
        elif isinstance(pymod, type) and issubclass(pymod, He):
            obj = Hs.__new__(cls, u, parse_mod_path_by_pycls(pymod))
        elif isinstance(pymod, ModuleType):
            obj = Hs.__new__(cls, u, pymod.__name__)
        else:
            raise ValueError(f'ModType 的参数 pymod 只允许： He | types.ModuleType | type(He)')
        return obj

    def __init__(self, u: He, pymod: ModuleType | type | He):
        if isinstance(pymod, He):
            Hs.__init__(self, u, parse_mod_path_by_pyobj(pymod))
            self.__x__['pycls'] = pymod.__class__
            self.__x__['pymod'] = __import__(pymod.__module__, fromlist=[''])
        elif isinstance(pymod, type) and issubclass(pymod, He):
            Hs.__init__(self, u, parse_mod_path_by_pycls(pymod))
            self.__x__['pycls'] = pymod
            self.__x__['pymod'] = __import__(pymod.__module__, fromlist=[''])
        elif isinstance(pymod, ModuleType):
            Hs.__init__(self, u, pymod.__name__)
            self.__x__['pymod'] = pymod

    @property
    @parse_self_x_item_value()
    def pymod(self): return
    @property
    @parse_self_x_item_value_or_set_by_func()
    def pycls(self):
        try: return import_pycls_by_mod_path(self)
        except: return None

    @property
    @parse_self_x_item_value_or_set_by_func()
    def mod_path(self): return self.__sm__.mod_path(self)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def cls_type(self):
        if self.pycls: return self.__sm__.cls_type(self.pycls)
        return None

    def rel_mod_path(self, rel_path=None): return self.mod_path.rel_mod_path(rel_path)
    def rel_mod_type(self, rel_path=None): return self.rel_mod_path(rel_path).mod_type

    @property
    @parse_self_x_item_value_or_set_by_func()
    def folder(self): return self.__sm__.folder(parse_dirpath_by_pymod(self.pymod))

    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_folder(self): return self.folder.child_folder(self.__s__.__i__.DIRNAM_DATA)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_configs_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_DATA_CONFIGS)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_configs_file(self): return self.data_configs_folder.child_file(self.__s__.__i__.FILNAM_CONFIGS)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_database_tables_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_DATA_DATABASE_TABLES)
    def data_database_table_file(self, name): return self.data_database_tables_folder.child_file(f'{name}.json')
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_database_data_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_DATA_DATABASE_DATA)
    def data_database_data_file(self, name): return self.data_database_data_folder.child_file(f'{name}.json')
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_multilinguals_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_DATA_MULTILINGALS)
    def data_multilinguals_file(self, language): return self.data_multilinguals_folder.child_file(f'{language}.json')

    @property
    @parse_self_x_item_value_or_set_by_func()
    def logs_folder(self): return self.folder.child_folder(self.__s__.__i__.DIRNAM_LOGS)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def public_folder(self): return self.folder.child_folder(self.__s__.__i__.DIRNAM_PUBLIC)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def multilinguals(self): return self.__sm__.multilinguals(folder=self.data_multilinguals_folder, hooked_configs=self.__sc__)
    def ml(self, code, language=None, replace=None, return_code=None): return self.multilinguals.ml(code, language, replace, return_code)
    def ml_name(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_name(language, replace, return_code)
    def ml_abbreviation(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_abbreviation(language, replace, return_code)
    def ml_title(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_title(language, replace, return_code)
    def ml_description(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_description(language, replace, return_code)
    def ml_link_title(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_link_title(language, replace, return_code)
    def ml_page_title(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_page_title(language, replace, return_code)
    def ml_breadcrumb_title(self, language=None, replace=None, return_code=None): return self.multilinguals.ml_breadcrumb_title(language, replace, return_code)



from .components import Components
from .configs import Configs
from .debugs import Debugs
from .i import I
from .types import Types
from .types.he import He


class System(He):
    def __init__(self): ...
    @property
    def __i__(self)-> I: ...
    @property
    def components(self)-> Components: ...
    @property
    def configs(self)-> Configs: ...
    @property
    def debugs(self)-> Debugs: ...
    @property
    def types(self)-> Types: ...

    def create_website(self): ...



from dbutils.pooled_db import PooledDB
from pymysql.cursors import DictCursor

from ....database import Database
from ....database.connection import Connection


class Mysql(Connection):
    @property
    def __u__(self)-> Database: ...
    @property
    def __pool__(self)-> PooledDB: ...
    @property
    def __is_connected__(self)-> bool: ...

    def query(self, sql: str)-> DictCursor | None: ...


from ..wrap import Wrap
from ... import Htmdoc
from ...node import Node


class Text(Node):
    def __init__(self, u: Htmdoc | Wrap, content:str=None): ...
    @property
    def content(self)-> str: ...



from ...nodes.tag import Tag


class Button(Tag):
    def __init__(self, u, title=None, attributes:dict[str, str|int|float|None]=None):
        if isinstance(attributes, dict):
            if 'type' not in attributes:
                attributes['type'] = 'button'
        else: attributes = {'type':'button'}
        Tag.__init__(self, u, None, title, attributes)



from .. import Debugs
from ....types.he import He


class Input(He):
    @property
    def __u__(self)-> Debugs: ...
    def input(self, label=None, prefix=None, color=None, bgcolor=None, bright=None)-> str: ...
    def primary(self, label=None, prefix=None)-> str: ...
    def secondary(self, label=None, prefix=None)-> str: ...
    def success(self, label=None, prefix=None)-> str: ...
    def warning(self, label=None, prefix=None)-> str: ...
    def danger(self, label=None, prefix=None)-> str: ...
    def info(self, label=None, prefix=None)-> str: ...
    def comment(self, label=None, prefix=None)-> str: ...




from types import ModuleType
from typing import Callable
from ..system.types.he import He


class HeEro(Exception, list):
    def __init__(self, he:type|He=None, func:str|Callable=None, feature:str=None, dec:str=None, dec_kwargs:dict[str, str|int|bool|None]=None, error:str|list[str]=None):
        self.es:list[str] = ...
        self.he:type = ...
        self.dec = ...
        self.dec_kwargs:dict = ...
        self.func = ...
        self.feature:str = ...
    def add(self, *e:str|Exception): ...

def parse_attribute_by_path_name(pyobj: object, name: str): ...

def join_url(*names) -> str: ...
def join_mod_path(*names) -> str: ...

def parse_pycls_name_by_mod_path(modnam: str) -> str: ...
def parse_mod_path_by_rel_path(mod_path: str, rel_path: str)-> str: ...
def parse_dirpath_by_pymod(pymod: ModuleType)-> str: ...
def parse_mod_path_by_pycls(pycls: type, *sub_names) -> str: ...
def parse_mod_path_by_pyobj(pyobj: object, *sub_names) -> str: ...
def parse_dirpath_by_pycls(cls: type) -> str: ...
def parse_dirpath_by_pyobj(obj: He) -> str: ...
def import_pymod_by_mod_path(module_path: str) -> ModuleType | None: ...
def import_pycls_by_mod_path(module_path: str) -> type | None: ...
def instance_pyobj_by_pycls(pycls: type, args=None, kwargs=None) -> He | object | None: ...
def instance_pyobj_by_mod_path(module_path: str, args=None, kwargs=None) -> He | object | None: ...
def import_webapp_pycls_by_mod_path(module_path: str) -> type | None: ...

def timestamp_days()-> int: ...
def timestamp_hours()-> int: ...
def timestamp_seconds()-> int: ...
def timestamp_milliseconds()-> int: ...
def timestamp_microseconds()-> int: ...
def timestamp_nanoseconds()-> int: ...


####################################################################################################################
BASE64URL: str = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-_"
def base_convert(value: str, base_from: int = 10, base_to: int = 32): ...
def uid_generater(prefix: str=None)-> str: ...
# COMPRESS_ID 字符必须相等长度
COMPRESS_ID_STR: str = 'ST'
COMPRESS_ID_INT: str = 'IN'
COMPRESS_ID_FLT: str = 'FT'
COMPRESS_ID_DIT: str = 'DT'
COMPRESS_ID_LST: str = 'LT'
COMPRESS_ID_BOL: str = 'BL'
COMPRESS_ID_NON: str = 'NO'
def compress(data: dict | list | int | float | bool | str | None) -> str: ...
def decompress(data: str) -> dict | list | int | float | bool | str | None: ...
def var2ui(value)-> str: ...



import os.path

from hebill.builtin.functions.builtins import HeEro
from ..file_folder import FileFolder


class Folder(FileFolder):
    def __init__(self, u, path: str):
        super().__init__(u, path)

    def is_exists(self): return os.path.isdir(self)
    def is_unexists(self): return not self.is_exists()
    def create(self): return os.makedirs(self, exist_ok=True)

    def child_file(self, name: str | list[str]): return self.__sm__.file(os.path.join(self, *(name if isinstance(name, list) else [name])))
    def child_folder(self, name: str | list[str]): return self.__sm__.folder(os.path.join(self, *(name if isinstance(name, list) else [name])))
    def search_children_names(self): return os.listdir(self) if self.is_exists() else []
    def search_children_file_names(self): return [name for name in self.search_children_names() if os.path.isfile(os.path.join(self, name))]
    def search_children_folder_names(self): return [name for name in self.search_children_names() if os.path.isdir(os.path.join(self, name))]
    def search_usable_children_names(self): return [name for name in self.search_children_names() if not name.startswith('.') and not name.startswith('_')]
    def search_usable_children_file_names(self): return [name for name in self.search_children_file_names() if not name.startswith('.') and not name.startswith('_')]
    def search_usable_children_folder_names(self): return [name for name in self.search_children_folder_names() if not name.startswith('.') and not name.startswith('_')]

    def search_children_paths(self): return [os.path.join(self, name) for name in self.search_children_names()]
    def search_children_file_paths(self): return [os.path.join(self, name) for name in self.search_children_file_names()]
    def search_children_folder_paths(self): return [os.path.join(self, name) for name in self.search_children_folder_names()]
    def search_usable_children_paths(self): return [os.path.join(self, name) for name in self.search_usable_children_names()]
    def search_usable_children_file_paths(self): return [os.path.join(self, name) for name in self.search_usable_children_file_names()]
    def search_usable_children_folder_paths(self): return [os.path.join(self, name) for name in self.search_usable_children_folder_names()]

    def search_children_files(self): return [self.child_file(name) for name in self.search_children_file_names()]
    def search_children_folders(self): return [self.child_folder(name) for name in self.search_children_folder_names()]
    def search_usable_children_files(self): return [self.child_file(name) for name in self.search_usable_children_file_names()]
    def search_usable_children_folders(self): return [self.child_folder(name) for name in self.search_usable_children_folder_names()]
    def search_children(self): return self.search_children_files() + self.search_children_folders()
    def search_usable_children(self): return self.search_usable_children_files() + self.search_usable_children_folders()

    def copyto(self, target_folder):
        e = HeEro(self, self.copyto, '将目录目录复制到指定目录。')
        if not isinstance(target_folder, Folder): target_folder = FileFolder(self.__s__, target_folder)
        if target_folder.is_exists(): raise e.add(f'目录：{target_folder} 已经存在！')
        target_folder.create()
        import shutil
        from pathlib import Path
        src = Path(self)
        dst = Path(target_folder)
        # 复制“目录内部内容”
        for item in src.iterdir():
            target = dst / item.name
            if item.is_dir(): shutil.copytree(item, target)
            else: shutil.copy2(item, target)



from ......types.he import He
from .column import Column


class Columns(He):
    @property
    def __database__(self): return self.__table__.__database__
    @property
    def __tables__(self): return self.__table__.__tables__
    @property
    def __table__(self): return self.__u__
    @property
    def __connection__(self): return self.__database__.connection
    @property
    def __configs__(self): return self.__database__.configs

    def column(self, name): return Column(self, name)

    def search_names(self, keyword=None, startwith=None, endwith=None): return self.__connection__.search_table_columns(self.__table__.fullname, keyword, startwith, endwith)



from ....types.he import He


class Webapp(He):
    def __init__(self, webmod):
        super().__init__(webmod)

    @property
    def x(self): return self.__u__

    def page_iterative(self): pass
    def page_content(self): pass

    def page_layout(self): pass

    def page_block_navbar(self): pass
    def page_block_breadcrumb(self): pass
    def page_block_navigations(self): pass
    def page_block_messages(self): pass
    def page_block_footer(self): pass
    def page_block_debugs(self): pass

    def page_block_1010(self): pass
    def page_block_1020(self): pass
    def page_block_1030(self): pass
    def page_block_1040(self): pass
    def page_block_1050(self): pass
    def page_block_1060(self): pass
    def page_block_1070(self): pass
    def page_block_1080(self): pass
    def page_block_1090(self): pass
    def page_block_1100(self): pass



from .....multilinguals import Multilinguals as ModMultilinguals


class Multilinguals(ModMultilinguals):
    def __init__(self, u):
        super().__init__(u, folder=u.data_multilinguals_folder)
    # @@@@@@@@@！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！
    # 由于 webmod 在类中缓存了，所以一单在 __init__ 中绑定页面刷新就不会重新实例化，但是系统一直运行，这里重写函数
    @property
    def ___hooked_configs___(self): return self.__u__.website.client.request.address



from ...__data__ import Data


class Select(Data):
    @property
    def columns(self): return self.__columns__
    @property
    def conditions(self): return self.__conditions__
    @property
    def orders(self): return self.__orders__
    @property
    def limits(self): return self.__limits__

    def execute(self): return self.__queries__.select_from_table(self.__table__.fullname, self.columns, self.conditions, self.orders, self.limits)



from ..file import File
from ...types.hd import Hd
from ...types.he import He


class Configs(Hd):
    def __init__(self, u: He, data: dict = None, end_class: type = None, start_class: type = None, file: str | File = None, defaults:dict=None): ...

    @property
    def ___value_types___(self)-> dict[str, str]: ...

    @property
    def ___hooks___(self)-> dict[str, tuple[Configs, str]]: ...
    def hook(self, name: str, target_configs: Configs, target_name: str = None): ...

    @property
    def ___defaults___(self) -> Configs: ...

    def __getitem__(self, name): ...
    def __setitem__(self, name: str, value): ...
    def get(self, name): ...
    def set(self, name: str, value): ...


    @property
    def ___file___(self)-> File | None: ...

    def save(self): ...



from ......types.he import He
from .... import Database
from ... import Tables
from .. import Table
from .column import Column
from ....connection import Connection
from ....configs import Configs


class Columns(He):
    @property
    def __database__(self)-> Database: ...
    @property
    def __tables__(self)-> Tables: ...
    @property
    def __table__(self)-> Table: ...
    @property
    def __connection__(self)-> Connection: ...
    @property
    def __configs__(self)-> Configs: ...

    def column(self, name)-> Column: ...
    def search_names(self, keyword=None, startwith: bool = None, endwith: bool = None) -> list[str]: ...



from ..he import He
from .number import Number


class Hf(He, float, Number):
    def __new__(cls, u, number: str | float | int):
        if isinstance(number, str):
            try: number = float(number.replace(',', '').replace(' ', ''))
            except: raise RuntimeError('实例化 Hn 实例参数 number 必须支持 float(number)。')
        if not isinstance(number, float | int):
            raise RuntimeError('实例化 Hn 实例参数 number 只支持 str | float | int 。')
        # noinspection PyTypeChecker
        obj = float.__new__(cls, number)
        obj.u = u
        return obj

    def __init__(self, u, number: str | float | int):
        He.__init__(self, u)




from .condition import Condition

class Conditions(list):
    def __init__(self, logic: str=None, is_root: bool=True):
        self._logic: str = ...
        self._is_root: bool = ...

    @property
    def is_root(self) -> bool: ...
    @property
    def logic(self) -> str: ...
    @logic.setter
    def logic(self, value) -> None: ...
    def set_logic_and(self) -> None: ...
    def set_logic_or(self) -> None: ...
    def add(self, column:str, value: str | int | float, judge: str=None, logic:str=None, startwith:bool=None, endwith:bool=None) -> Condition: ...
    def add_and_equal(self, column:str, value: str | int | float) -> Condition: ...
    def add_or_equal(self, column:str, value: str | int | float) -> Condition: ...
    def add_and_not_equal(self, column:str, value: str | int | float) -> Condition: ...
    def add_and_more_than(self, column:str, value: str | int | float) -> Condition: ...
    def add_and_not_more_than(self, column:str, value: str | int | float) -> Condition: ...
    def add_and_less_than(self, column:str, value: str | int | float) -> Condition: ...
    def add_and_not_less_than(self, column:str, value: str | int | float) -> Condition: ...
    def add_and_like(self, column:str, value: str | int | float, startwith:bool=None, endwith:bool=None) -> Condition: ...
    def add_or_like(self, column:str, value: str | int | float, startwith:bool=None, endwith:bool=None) -> Condition: ...
    def add_conditions(self, logic:str=None) -> Conditions: ...
    def add_conditions_and(self) -> Conditions: ...
    def add_conditions_or(self) -> Conditions: ...
    def output(self, logic_bridge_requeried:bool=False) -> str: ...



from .cls import Cls
from .med import Med
from ..file import File
from ...types.he import He


class CssMaker(He):
    def __init__(self, u: He, file: str | File = None): ...

    @property
    def file(self)-> File | None: ...
    @property
    def version(self)-> str: ...

    @property
    def sizes(self)-> list[str]: ...
    @sizes.setter
    def sizes(self, value: list[str]): return

    @property
    def cls_root(self)-> Cls: ...
    @property
    def css_html(self)-> Cls: ...
    @property
    def css_body(self)-> Cls: ...

    @property
    def clses(self)-> dict[str, Cls]: ...
    def cls(self, name:str, media_min_width:int=None, media_max_width:int=None, media_min_height:int=None, media_max_height:int=None)-> Cls: ...

    @property
    def medes(self)-> dict[str, Cls]: ...
    def med(self, media_min_width:int=None, media_max_width:int=None, media_min_height:int=None, media_max_height:int=None)-> Med: ...

    def save(self, minimized: bool = None): ...


import re

from ......functions.builtins import import_webapp_pycls_by_mod_path
from ......functions.decorators import parse_self_x_item_value_or_set_by_func, \
    parse_self_x_item_value_or_set_by_instance_he_by_module_name
from .....types.hs import Hs


class Webmod(Hs):
    def __init__(self, u, fullname: str):
        super().__init__(u, fullname)
        self.__x__['fullname'] = fullname

    @property
    def websystem(self): return self.website.websystem
    """@property
    def server(self): return self.websystem.server"""
    @property
    def website(self): return self.__u__
    """@property
    def client(self): return self.website.client
    @property
    def database(self): return self.websystem.database"""

    def webmod(self, *names: str): return self.website.webmod(*names)
    @property
    def webmod_home(self): return self.webmod()
    @property
    def webmod_page_content_sign_in(self): return self.webmod(self.websystem.configs.webmod_page_content_sign_in)
    @property
    def webmod_page_content_sign_out(self): return self.webmod(self.websystem.configs.webmod_page_content_sign_out)
    @property
    def webmod_page_content_sign_up(self): return self.webmod(self.websystem.configs.webmod_page_content_sign_up)
    @property
    def webmod_page_content_about(self): return self.webmod(self.websystem.configs.webmod_page_content_about)
    @property
    def webmod_page_content_contact(self): return self.webmod(self.websystem.configs.webmod_page_content_contact)
    @property
    def webmod_page_content_admin(self): return self.webmod(self.websystem.configs.webmod_page_content_admin)

    @property
    def webmod_page_layout(self): return self.webmod(self.websystem.configs.webmod_page_layout)
    @property
    def webmod_page_block_navbar(self): return self.webmod(self.websystem.configs.webmod_page_block_navbar)
    @property
    def webmod_page_block_breadcrumb(self): return self.webmod(self.websystem.configs.webmod_page_block_breadcrumb)
    @property
    def webmod_page_block_navigations(self): return self.webmod(self.websystem.configs.webmod_page_block_navigations)
    @property
    def webmod_page_block_messages(self): return self.webmod(self.websystem.configs.webmod_page_block_messages)
    @property
    def webmod_page_block_footer(self): return self.webmod(self.websystem.configs.webmod_page_block_footer)
    @property
    def webmod_page_block_debugs(self): return self.webmod(self.websystem.configs.webmod_page_block_debugs)

    # 按照 setting 中 serial 排序
    def search_children_webmods(self):
        return [self.child_webmod(i) for i in self.folder.search_usable_children_folder_names()]
        # return sorted([self.child_webmod(i) for i in self.folder.search_usable_children_folder_names()], key=lambda i: i.settings.serial)
    def search_children_webmods_by_page_featur_general(self):
        return [i for i in self.search_children_webmods() if i.settings.is_page_feature_general()]

    def has_page_iterative(self): return self.has_local_method('page_iterative')
    def has_page_content(self): return self.has_local_method('page_content')
    def has_page_layout(self): return self.has_local_method('page_layout')
    def has_page_block_navbar(self): return self.has_local_method('page_block_navbar')
    def has_page_block_breadcrumb(self): return self.has_local_method('page_block_breadcrumb')
    def has_page_block_navigations(self): return self.has_local_method('page_block_navigations')
    def has_page_block_messages(self): return self.has_local_method('page_block_messages')
    def has_page_block_footer(self): return self.has_local_method('page_block_footer')
    def has_page_block_debugs(self): return self.has_local_method('page_block_debugs')
    def has_page_block_1010(self): return self.has_local_method('page_block_1010')
    def has_page_block_1020(self): return self.has_local_method('page_block_1020')
    def has_page_block_1030(self): return self.has_local_method('page_block_1030')
    def has_page_block_1040(self): return self.has_local_method('page_block_1040')
    def has_page_block_1050(self): return self.has_local_method('page_block_1050')
    def has_page_block_1060(self): return self.has_local_method('page_block_1060')
    def has_page_block_1070(self): return self.has_local_method('page_block_1070')
    def has_page_block_1080(self): return self.has_local_method('page_block_1080')
    def has_page_block_1090(self): return self.has_local_method('page_block_1090')
    def has_page_block_1100(self): return self.has_local_method('page_block_1100')

    @property
    @parse_self_x_item_value_or_set_by_func()
    def senior(self): return None if self.is_root() else self.webmod(*self.split('.')[:-1])

    def child_webmod(self, *names): return self.website.webmod(self, *names) if names else None

    @property
    @parse_self_x_item_value_or_set_by_func()
    def name(self): return self.tree_names[-1]
    @parse_self_x_item_value_or_set_by_func()
    def is_name_serialized(self): return bool(re.match(r'^n\d+_', self.name))
    @property
    @parse_self_x_item_value_or_set_by_func()
    def net_name(self): return self.name.split('_', 1)[-1] if self.is_name_serialized() else self.name
    @property
    @parse_self_x_item_value_or_set_by_func()
    def tree_names(self): return self.split('.')

    @property
    @parse_self_x_item_value_or_set_by_func()
    def fullname(self): return

    def is_root(self): return self == ''

    @property
    @parse_self_x_item_value_or_set_by_func()
    def tree(self): return [self] if self.is_root() else [*self.senior.tree, self]

    @property
    @parse_self_x_item_value_or_set_by_func()
    def mod_path(self): return self.__sm__.mod_path(self.websystem.configs.module if self.is_root() else '.'.join([self.websystem.configs.module, self.fullname]))
    @property
    def mod_type(self): return self.mod_path.mod_type
    def is_mod_exists(self): return self.mod_type.pymod is not None
    @property
    @parse_self_x_item_value_or_set_by_func()
    def cls_type(self):
        try: return self.__sm__.cls_type(import_webapp_pycls_by_mod_path(self.mod_path))
        except: return None
    def is_cls_exists(self): return self.cls_type is not None
    def has_local_method(self, name): return self.is_cls_exists() and callable(self.cls_type.pycls.__dict__.get(name))

    @property
    @parse_self_x_item_value_or_set_by_func()
    def obj_type(self):
        if self.is_cls_exists():
            try: return self.__sm__.obj_type(self.cls_type.pycls(self))
            except: pass
        return None
    def is_object_instantiated(self): return self.obj_type is not None

    @property
    def folder(self): return self.websystem.website_folder if self.is_root() else self.senior.folder.child_folder(self.name)
    def is_folder_exists(self): return self.folder.is_exists()
    def folder_child_file(self, name: str | list[str]): return self.folder.child_file(name)
    def folder_child_folder(self, name: str | list[str]): return self.folder.child_folder(name)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_folder(self): return self.folder.child_folder(self.__s__.__i__.DIRNAM_DATA)
    def data_folder_child_file(self, name: str | list[str]): return self.data_folder.child_file(name)
    def data_folder_child_folder(self, name: str | list[str]): return self.data_folder.child_folder(name)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_configs_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_DATA_CONFIGS)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_configs_file(self): return self.data_configs_folder.child_file(self.__s__.__i__.FILNAM_CONFIGS)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_settings_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_DATA_SETTINGS)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_settings_file(self): return self.data_settings_folder.child_file(self.__s__.__i__.FILNAM_SETTINGS)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_database_tables_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_DATA_DATABASE_TABLES)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_database_data_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_DATA_DATABASE_DATA)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def data_multilinguals_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_DATA_MULTILINGALS)
    def data_multilinguals_file(self, language: str): return self.data_multilinguals_folder.child_file(f'{language}.json')
    @property
    @parse_self_x_item_value_or_set_by_func()
    def logs_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_LOGS)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def public_folder(self): return self.data_folder.child_folder(self.__s__.__i__.DIRNAM_PUBLIC)
    def public_folder_child_file(self, name: str | list[str]): return self.public_folder.child_file(name)
    def public_folder_child_folder(self, name: str | list[str]): return self.public_folder.child_folder(name)
    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def multilinguals(self): return
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___ml_by_webmod_name___(self): return ' '.join([i.capitalize() for i in self.net_name.split('_') if i])
    def ml(self, code, language=None, replace=None, return_code=True): return (r:=self.multilinguals.ml(code, language, replace, False)) or (self.___ml_by_webmod_name___ if return_code else r)
    def ml_name(self, language=None, replace=None, return_code=True): return (r:=self.multilinguals.ml_name(language, replace, False)) or (self.___ml_by_webmod_name___ if return_code else r)
    def ml_abbreviation(self, language=None, replace=None, return_code=True): return (r:=self.multilinguals.ml_abbreviation(language, replace, False)) or (self.___ml_by_webmod_name___ if return_code else r)
    def ml_title(self, language=None, replace=None, return_code=True): return (r:=self.multilinguals.ml_title(language, replace, False)) or (self.___ml_by_webmod_name___ if return_code else r)
    def ml_description(self, language=None, replace=None, return_code=True): return (r:=self.multilinguals.ml_description(language, replace, False)) or (self.___ml_by_webmod_name___ if return_code else r)
    def ml_link_title(self, language=None, replace=None, return_code=True): return (r:=self.multilinguals.ml_link_title(language, replace, False)) or (self.___ml_by_webmod_name___ if return_code else r)
    def ml_page_title(self, language=None, replace=None, return_code=True): return (r:=self.multilinguals.ml_page_title(language, replace, False)) or (self.___ml_by_webmod_name___ if return_code else r)
    def ml_breadcrumb_title(self, language=None, replace=None, return_code=True): return (r:=self.multilinguals.ml_breadcrumb_title(language, replace, False)) or (self.___ml_by_webmod_name___ if return_code else r)

    @property
    def htmdoc(self): return self.website.htmdoc

    @property
    def page_areas(self): return self.website.htmdoc.areas

    @property
    def page_area_content(self): return self.website.htmdoc.areas.content
    @property
    def page_area_navbar(self): return self.website.htmdoc.areas.navbar
    @property
    def page_area_breadcrumb(self): return self.website.htmdoc.areas.breadcrumb
    @property
    def page_area_navigations(self): return self.website.htmdoc.areas.navigations
    @property
    def page_area_messages(self): return self.website.htmdoc.areas.messages
    @property
    def page_area_footer(self): return self.website.htmdoc.areas.footer
    @property
    def page_area_debugs(self): return self.website.htmdoc.areas.debugs

    def __execute__(self, name):
        # getattr(self.pyobject, name)(); return True
        if self.is_object_instantiated():
            if self.is_object_instantiated():
                # getattr(self.pyobject, name)(); return True
                try: getattr(self.obj_type, name)(); return True
                except Exception as e: self.websystem.server.__deubg_warning__(f'运行模块失败', e, f'{self.mod_path}.core.Core().{name}()', __file__)
        return False

    def execute_page_iterative(self):
        for m in self.tree:
            if m.has_page_iterative():
                if not m.__execute__('page_iterative'):
                    return False
        return True
    def execute_page_content(self): return self.__execute__('page_content')
    def execute_page_layout(self): return self.__execute__('page_layout')
    def execute_page_block_navbar(self): return self.__execute__('page_block_navbar')
    def execute_page_block_breadcrumb(self): return self.__execute__('page_block_breadcrumb')
    def execute_page_block_navigations(self): return self.__execute__('page_block_navigations')
    def execute_page_block_messages(self): return self.__execute__('page_block_messages')
    def execute_page_block_footer(self): return self.__execute__('page_block_footer')
    def execute_page_block_debugs(self): return self.__execute__('page_block_debugs')
    def execute_page_block_1010(self): return self.__execute__('page_block_1010')
    def execute_page_block_1020(self): return self.__execute__('page_block_1020')
    def execute_page_block_1030(self): return self.__execute__('page_block_1030')
    def execute_page_block_1040(self): return self.__execute__('page_block_1040')
    def execute_page_block_1050(self): return self.__execute__('page_block_1050')
    def execute_page_block_1060(self): return self.__execute__('page_block_1060')
    def execute_page_block_1070(self): return self.__execute__('page_block_1070')
    def execute_page_block_1080(self): return self.__execute__('page_block_1080')
    def execute_page_block_1090(self): return self.__execute__('page_block_1090')
    def execute_page_block_1100(self): return self.__execute__('page_block_1100')

    def address(self, mode=None, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.website.client.address(mode, self.fullname, arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security)
    def url(self, mode=None, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.address(mode, arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security).output()
    def redirect(self, mode=None, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.address(mode, arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security).redirect()

    def page_address(self, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.address('page', arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security)
    def page_url(self, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.page_address(arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security).output()
    def page_redirect(self, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.page_address(arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security).redirect()


    def address_by_current(self): return self.website.client.request.address.copy()
    def url_by_current(self): return self.address_by_current().output()
    def address_by_current_change_language(self, language=None):
        a = self.address_by_current()
        a.language = language
        return a
    def url_by_current_change_language(self, language=None): return self.address_by_current_change_language(language).output()
    def address_by_current_change_pagination(self, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None):
        a = self.address_by_current()
        if keyword is not None: a.keyword = keyword
        a.page, a.orders, a.filters, a.items_per_page, a.pages_per_group = page, orders, filters, items_per_page, pages_per_group
        return a
    def url_by_current_change_pagination(self, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None): return self.address_by_current_change_pagination(keyword, page, orders, filters, items_per_page, pages_per_group).output()

    def set_page_navbar_logo(self, url): return
    def add_page_breadcrumb(self, title=None, url=None): self.htmdoc.caches.breadcrumb.add_menu(title, url)

    def public_file_url(self, *names: str): return self.websystem.server.generate_url(*self.split('.'), self.__s__.__i__.DIRNAM_PUBLIC, *names)

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def settings(self): return

    def is_active_by_client(self): return self.website.client.request.address.module == self



from decimal import Decimal
from ..hf.number import Number
from ..he import He


class Hm(He, Decimal, Number):
    def __new__(cls, u: He, number: str | float | int): ...
    def __init__(self, u: He, number: str | float | int): ...



class Menu(dict):
    def __init__(self, title=None, url=None): ...
    @property
    def title(self): ...
    @title.setter
    def title(self, value): ...
    @property
    def url(self): ...
    @url.setter
    def url(self, value): ...



from ........functions.decorators import parse_self_x_item_value
from .......types.he import He


class Column(He):
    def __init__(self, u, name):
        He.__init__(self, u)
        self.__x__['name'] = name
    @property
    def __database__(self): return self.__columns__.__database__
    @property
    def __tables__(self): return self.__columns__.__tables__
    @property
    def __table__(self): return self.__columns__.__table__
    @property
    def __columns__(self): return self.__u__
    @property
    def __connection__(self): return self.__database__.connection
    @property
    def __configs__(self): return self.__database__.configs

    @property
    @parse_self_x_item_value()
    def name(self): return

    def exists(self): return self.name in self.__columns__.search_names()
    def create(self, datatype=None, length=0, unique=False, nullable=False): return self.__connection__.create_table_column(self.__table__.fullname, self.name, datatype, length, unique, nullable)
    def delete(self): return

    def set_unique(self): return self.__connection__.set_table_column_quniue(self.__table__.fullname, self.name)



from .....multilinguals import Multilinguals as ModMultilinguals
from .. import Webmod


class Multilinguals(ModMultilinguals):
    def __init__(self, u: Webmod): ...
    @property
    def __u__(self)-> Webmod: ...



from ...__data__ import Data
from .....queries.plugins.columns import Columns
from .....queries.plugins.conditions import Conditions
from .....queries.plugins.limits import Limits
from .....queries.plugins.orders import Orders


class Select(Data):
    @property
    def columns(self) -> Columns: ...
    @property
    def conditions(self) -> Conditions: ...
    @property
    def orders(self) -> Orders: ...
    @property
    def limits(self) -> Limits: ...

    def execute(self) -> list[dict]: ...




from ......functions.decorators import parse_self_x_item_value, \
    parse_self_x_item_value_or_set_by_instance_he_by_module_name
from .....types.he import He


class Table(He):
    def __init__(self, u, name):
        He.__init__(self, u)
        self.__x__['name'] = name
    @property
    def __database__(self): return self.__tables__.__database__
    @property
    def __tables__(self): return self.__u__
    @property
    def __connection__(self): return self.__database__.connection
    @property
    def __configs__(self): return self.__database__.configs
    @property
    @parse_self_x_item_value()
    def name(self): return
    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def columns(self): return
    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def data(self): return
    @property
    @parse_self_x_item_value()
    def fullname(self): return f'{self.__configs__['prefix']}{self.name}'

    def exists(self): return self.name in self.__tables__.search_names()
    def create(self): return self.__connection__.create_table(self.fullname)
    def delete(self): return self.__connection__.delete_table(self.fullname)


from .. import Debugs
from ....types.he import He


class Output(He):
    @property
    def __u__(self)-> Debugs: ...
    def print(self, content=None, error=None, comment=None, file=None, prefix=None, color=None, bgcolor=None, bright=None, subsystem=None)-> None: ...
    def primary(self, content=None, error=None, comment=None, file=None, prefix=None)-> None: ...
    def secondary(self, content=None, error=None, comment=None, file=None, prefix=None)-> None: ...
    def success(self, content=None, error=None, comment=None, file=None, prefix=None)-> None: ...
    def warning(self, content=None, error=None, comment=None, file=None, prefix=None)-> None: ...
    def danger(self, content=None, error=None, comment=None, file=None, prefix=None)-> None: ...
    def info(self, content=None, error=None, comment=None, file=None, prefix=None)-> None: ...
    def comment(self, content=None, error=None, comment=None, file=None, prefix=None)-> None: ...



from datetime import datetime, tzinfo

from ....functions.decorators import parse_dict_item_value_or_set_by_func
from .....builtin.functions.decorators import parse_self_x_item_value_or_set_by_func


class Number:
    CN_INTEGER_0_TO_9 = '零壹贰叁肆伍陆柒捌玖'
    CN_POINTS_UNITS = '角分厘毫丝'
    CN_DECIMAL_4S_UNITS = '万亿兆京垓秭穰沟涧正载极'
    CN_DECIMAL_UNITS = '拾佰仟'

    CN_CURRENCY_UNIT = "圆"
    CN_POINT_NAME = '点'
    CN_ONLY_NAME = '整'
    CN_CNY_NAME = '人民币'
    CN_USD_NAME = '美元'
    CN_EUR_NAME = '欧元'
    CN_MINUS = '负'

    EN_CNY_NAME = 'Say Chinese Yuan'
    EN_USD_NAME = 'Say U.S.Dollar'
    EN_EUR_NAME = 'Say Euro'
    EN_ONLY_NAME = 'Only'
    EN_POINT_NAME = 'Point'
    EN_AND_NAME = 'and'
    EN_MINUS = 'Minus'

    EN_INTEGER_0_TO_9 = ['Zero', 'One', 'Two', 'Three', 'Four', 'Five', 'Six', 'Seven', 'Eight', 'Nine']
    EN_INTEGER_1NS = ['Ten', 'Eleven', 'Twelve', 'Thirteen', 'Fourteen', 'Fifteen', 'Sixteen', 'Seventeen',
                      'Eighteen', 'Nineteen']
    EN_INTEGER_2NS = ['Twenty', 'Thirty', 'Forty', 'Fifty', 'Sixty', 'Seventy', 'Eighty', 'Ninety']
    EN_HUNDRED_NAME = 'Hundred'
    EN_DECIMAL_3S_UNITS = ['Thousand', 'Million', 'Billion', 'Trillion', 'Quadrillion', 'Quintillion', 'Sextillion',
                           'Septillion', 'Octillion', 'Nonillion', 'Decillion']
    EN_POINTS_UNITS = ['Dime', 'Cent', 'Mill']

    WEIGHT_UNIT_KGS = 'Kgs'
    CN_WEIGHT_UNIT_KGS = '公斤'
    EN_WEIGHT_UNIT_KGS = 'Kilograms'

    LENGTH_UNIT_MM = 'mm'
    CN_LENGTH_UNIT_MM = '毫米'
    EN_LENGTH_UNIT_MM = 'Millimetres'

    LENGTH_UNIT_M = 'm'
    CN_LENGTH_UNIT_M = '米'
    EN_LENGTH_UNIT_M = 'Metres'

    LENGTH_UNIT_CM = 'cm'
    CN_LENGTH_UNIT_CM = '厘米'
    EN_LENGTH_UNIT_CM = 'Centimeters'

    def is_int(self): return int(self) == self
    def is_positive(self): return self > 0
    def is_negative(self): return self < 0
    def is_zero(self): return self == 0

    def round(self, digits: int = 0):
        r = self.__round__(digits)
        if digits > 0: return self.__st__.hf(r)
        return self.__st__.hi(r)
    def ceil(self, digits: int = 0):
        r = (self / 10**(-digits)).__ceil__() * 10 **(-digits)
        if digits > 0: return self.__st__.hf(r)
        return self.__st__.hi(r)
    def floor(self, digits: int = 0):
        r = (self / 10**(-digits)).__floor__() * 10 **(-digits)
        if digits > 0: return self.__st__.hf(r)
        return self.__st__.hi(r)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def integer_part(self):
        return str(abs(int(self)))
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___integer_part_breaks___(self): return {}
    @parse_dict_item_value_or_set_by_func(dict_attr_name='___integer_part_breaks___', dict_item_key_by_arg=0)
    def integer_part_breaks(self, step: int): return [self.integer_part[max(i - step, 0):i] for i in range(len(self.integer_part), 0, -step)][::-1]
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___integer_part_strict_breaks___(self): return {}
    @parse_dict_item_value_or_set_by_func(dict_attr_name='___integer_part_strict_breaks___', dict_item_key_by_arg=0)
    def integer_part_strict_breaks(self, step: int): return [self.integer_part_breaks(step)[0].zfill(step), *self.integer_part_breaks(step)[1:]]

    @property
    @parse_self_x_item_value_or_set_by_func()
    def integer_part_myriad_breaks(self): return self.integer_part_breaks(4)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def integer_part_myriad_strict_breaks(self): return self.integer_part_strict_breaks(4)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def integer_part_thousand_breaks(self): return self.integer_part_breaks(3)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def integer_part_thousand_strict_breaks(self): return self.integer_part_strict_breaks(3)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def decimal_part(self): return '' if '.' not in (s:=str(self)) or (r:=s.split('.', 1)[1]) == '0' else r
    ####################################################################################################
    def round_format(self, digits: int = 0, force_digits: bool = None, integer_breaks: int = 3): return f'{','.join((n:=self.round(digits)).integer_part_breaks(integer_breaks))}{'.' + r if (r:=n.decimal_part.ljust(digits, '0') if digits > 0 and force_digits else n.decimal_part) else ''}'

    def year(self, tz: tzinfo = None): return datetime.fromtimestamp(self, tz).year
    def month(self, tz: tzinfo = None): return datetime.fromtimestamp(self, tz).month
    def day(self, tz: tzinfo = None): return datetime.fromtimestamp(self, tz).day
    def hour(self, tz: tzinfo = None): return datetime.fromtimestamp(self, tz).hour
    def minute(self, tz: tzinfo = None): return datetime.fromtimestamp(self, tz).minute
    def second(self, tz: tzinfo = None): return datetime.fromtimestamp(self, tz).second

    def format_yy(self, tz: tzinfo = None): return str(self.year(tz))[-2:]
    def format_m(self, tz: tzinfo = None): return str(self.month(tz))
    def format_d(self, tz: tzinfo = None): return str(self.day(tz))
    def format_h(self, tz: tzinfo = None): return str(self.hour(tz))
    def format_i(self, tz: tzinfo = None): return str(self.minute(tz))
    def format_s(self, tz: tzinfo = None): return str(self.second(tz))
    def format_yyyy(self, tz: tzinfo = None): return str(self.year(tz))
    def format_mm(self, tz: tzinfo = None): return str(self.month(tz)).rjust(2, '0')
    def format_dd(self, tz: tzinfo = None): return str(self.day(tz)).rjust(2, '0')
    def format_hh(self, tz: tzinfo = None): return str(self.hour(tz)).rjust(2, '0')
    def format_ii(self, tz: tzinfo = None): return str(self.minute(tz)).rjust(2, '0')
    def format_ss(self, tz: tzinfo = None): return str(self.second(tz)).rjust(2, '0')

    def format_yymd(self, tz: tzinfo = None, delimiter='-'): return f'{self.format_yy(tz)}{(d:=delimiter)}{self.format_m(tz)}{d}{self.format_d(tz)}'
    def format_his(self, tz: tzinfo = None, delimiter=':'): return f'{self.format_h(tz)}{(d:=delimiter)}{self.format_i(tz)}{d}{self.format_s(tz)}'
    def format_yymdhis(self, tz: tzinfo = None, delimiter=' ', date_delimiter='-', time_delimiter=':'): return f'{self.format_yymd(tz, date_delimiter)}{delimiter}{self.format_his(tz, time_delimiter)}'
    def format_yyyymmdd(self, tz: tzinfo = None, delimiter='-'): return f'{self.format_yyyy(tz)}{(dd:=delimiter)}{self.format_mm(tz)}{dd}{self.format_dd(tz)}'
    def format_hhiiss(self, tz: tzinfo = None, delimiter=':'): return f'{self.format_hh(tz)}{(td:=delimiter)}{self.format_ii(tz)}{td}{self.format_ss(tz)}'
    def format_yyyymmddhhiiss(self, tz: tzinfo = None, delimiter=' ', date_delimiter='-', time_delimiter=':'): return f'{self.format_yyyymmdd(tz, date_delimiter)}{delimiter}{self.format_hhiiss(tz, time_delimiter)}'

    ####################################################################################################
    # 以下数字转化为大写
    ####################################################################################################

    @property
    @parse_self_x_item_value_or_set_by_func()
    def cn_decimal_part(self):
        if not self.decimal_part: return ''
        cn = ''
        for i in range(0, len(self.decimal_part)):
            cn += self.CN_INTEGER_0_TO_9[int(self.decimal_part[i])]
        return cn

    def _capitalized_myriad_breaks_number_to_cn(self, group_sn, num_sn):
        num = int(self.integer_part_myriad_strict_breaks[group_sn][num_sn])
        cap = f'{self.CN_INTEGER_0_TO_9[int(self.integer_part_myriad_strict_breaks[group_sn][num_sn])]}'
        if num != 0 and num_sn < 3:
            cap += f'{self.CN_DECIMAL_UNITS[2 - num_sn]}'
        return cap

    @property
    @parse_self_x_item_value_or_set_by_func()
    def cn_integer_part(self):
        bs, qnt, cn, first, zero = self.integer_part_myriad_strict_breaks, len(self.integer_part_myriad_strict_breaks), '', True, False
        for i, block in enumerate(bs):
            part = ''
            for j, c in enumerate(block):
                if c != '0':
                    if zero and not first: part += self.CN_INTEGER_0_TO_9[0]
                    part += self.CN_INTEGER_0_TO_9[int(c)]
                    if j < 3: part += self.CN_DECIMAL_UNITS[2 - j]
                    first = zero = False
                else: zero = True
            cn += part
            if part and i < qnt - 1: cn += self.CN_DECIMAL_4S_UNITS[qnt - i - 2]
        if self.is_negative(): return f'{self.CN_MINUS}{cn or self.CN_INTEGER_0_TO_9[0]}'
        return cn or self.CN_INTEGER_0_TO_9[0]

    @property
    @parse_self_x_item_value_or_set_by_func()
    def cn_number(self): return f'{self.cn_integer_part}{self.CN_POINT_NAME}{self.cn_decimal_part}' if self.cn_decimal_part else self.cn_integer_part
    @property
    @parse_self_x_item_value_or_set_by_func()
    def cn_money(self):
        cn, dp = self.cn_integer_part + self.CN_CURRENCY_UNIT, self.decimal_part
        if not dp: return cn + self.CN_ONLY_NAME
        units, nums, tail = self.CN_POINTS_UNITS, self.CN_INTEGER_0_TO_9, ''
        for i in range(min(5, len(dp)) - 1, -1, -1):
            if dp[i] != '0' or tail: tail = nums[int(dp[i])] + units[i] + tail
        return cn + (tail or self.CN_ONLY_NAME)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def cn_cny(self): return self.CN_CNY_NAME + self.cn_money
    def cn_usd(self): return self.CN_USD_NAME + self.cn_money
    def cn_eur(self): return self.CN_EUR_NAME + self.cn_money
    # noinspection PyTypeHints
    @property
    @parse_self_x_item_value_or_set_by_func()
    def en_decimal_part(self): return ' '.join(self.EN_INTEGER_0_TO_9[int(n)] for n in self.decimal_part)
    def _capitalized_en_by_thousands_breaks_add_and(self, group_sn, number_sn):
        for i in range(number_sn + 1, 3):
            if self.integer_part_thousand_strict_breaks[group_sn][i] != '0': return True
        return False
    # noinspection PyTypeHints
    def _capitalized_en_by_thousands_breaks(self, group_sn):
        g = self.integer_part_thousand_strict_breaks[group_sn]
        caps = []
        if g[0] != '0':
            caps.append(self.EN_INTEGER_0_TO_9[int(g[0])])
            caps.append(self.EN_HUNDRED_NAME)
            if self._capitalized_en_by_thousands_breaks_add_and(group_sn, 0): caps.append(self.EN_AND_NAME)
            caps.extend(self._capitalized_en_by_thousands_breaks_2(group_sn))
        else: caps.extend(self._capitalized_en_by_thousands_breaks_2(group_sn))
        return caps
    # noinspection PyTypeHints
    def _capitalized_en_by_thousands_breaks_2(self, group_sn):
        g = self.integer_part_thousand_strict_breaks[group_sn]
        caps = []
        if int(g[1]) >= 2:
            caps.append(self.EN_INTEGER_2NS[int(g[1]) - 2])
            caps.extend(self._capitalized_en_by_thousands_breaks_1(group_sn))
        elif g[1] == '1': caps.append(self.EN_INTEGER_1NS[int(g[2])])
        else: caps.extend(self._capitalized_en_by_thousands_breaks_1(group_sn))
        return caps
    # noinspection PyTypeHints sd
    def _capitalized_en_by_thousands_breaks_1(self, group_sn):
        g = self.integer_part_thousand_strict_breaks[group_sn]
        caps = []
        if g[2] != '0': caps.append(self.EN_INTEGER_0_TO_9[int(g[2])])
        return caps
    # noinspection PyTypeHints
    @property
    @parse_self_x_item_value_or_set_by_func()
    def en_integer_part(self):
        bs, caps, q = self.integer_part_thousand_strict_breaks, [], len(self.integer_part_thousand_strict_breaks)
        for i, g in enumerate(bs):
            group_caps = self._capitalized_en_by_thousands_breaks(i)
            if not group_caps: continue  # 全 0 group 跳过，不加 unit
            caps.extend(group_caps)
            # 只在当前 group 有数字时加 unit
            if i < q - 1 and any(x != '0' for x in bs[i]): caps.append(self.EN_DECIMAL_3S_UNITS[q - i - 2])
        en = ' '.join(caps) or self.EN_INTEGER_0_TO_9[0]
        return f'{self.EN_MINUS} {en}' if self.is_negative() else en
    @property
    @parse_self_x_item_value_or_set_by_func()
    def en_number(self): return f'{self.en_integer_part} {self.EN_POINT_NAME} {self.en_decimal_part}' if self.en_decimal_part else self.en_integer_part
    @property
    @parse_self_x_item_value_or_set_by_func()
    def en_money(self):
        en = self.en_integer_part
        ds = []
        # 遍历小数部分，每位对应 EN_POINTS_UNITS
        for i, digit in enumerate(self.decimal_part[:3]):  # 只处理前三位
            n = int(digit)
            if n == 0 and not ds: continue  # 前导零且还没有加入其他小数
            # noinspection PyTypeHints
            unit = self.EN_POINTS_UNITS[i]
            if n > 1: unit += 's'
            ds.append(unit)
            # noinspection PyTypeHints
            ds.append(self.EN_INTEGER_0_TO_9[n])
        if ds:
            ds.reverse()  # 小数位从高到低
            en += f' {self.EN_AND_NAME} ' + ' '.join(ds)
        else: en += f' {self.EN_ONLY_NAME}'
        return en

    @property
    @parse_self_x_item_value_or_set_by_func()
    def en_cny(self): return self.EN_CNY_NAME + ' ' + self.en_money
    @property
    @parse_self_x_item_value_or_set_by_func()
    def en_usd(self): return self.EN_USD_NAME + ' ' + self.en_money
    @property
    @parse_self_x_item_value_or_set_by_func()
    def en_eur(self): return self.EN_EUR_NAME + ' ' + self.en_money
